@@ -1,21 +1,38 @@
-/**
- *
- * $Id: gra.c,v 1.3 1999/04/15 12:15:27 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: gra.c,v 1.7 2002/07/06 08:51:42 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: gra.c,v $
+ * Revision 1.7  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.6  2002/06/16 07:38:37  isizaka
+ * add GRAexpandmath
+ *
+ * Revision 1.5  2001/07/14 17:42:55  isizaka
+ * for 6.03.14
+ *
  * Revision 1.3  1999/04/15 12:15:27  isizaka
  * for release 6.03.01
  *
@@ -30,6 +47,7 @@
 
 #include <limits.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
@@ -37,6 +55,7 @@
 #include "nstring.h"
 #include "jstring.h"
 #include "mathfn.h"
+#include "mathcode.h"
 #include "gra.h"
 
 #define TRUE 1
@@ -279,7 +298,6 @@ void GRAreopen(int GC)
 
   if (GC<0) return;
   if (GC>=GRAClimit) return;
-  GRAend(GC);
   memfree(GRAClist[GC].linedash);
   memfree(GRAClist[GC].gdashlist);
   memfree(GRAClist[GC].textfont);
@@ -297,7 +315,7 @@ void GRAreopen(int GC)
   cpar[2]=GRAClist[GC].topm;
   cpar[3]=GRAClist[GC].width;
   cpar[4]=GRAClist[GC].height;
-  cpar[5]=round(GRAClist[GC].zoom*10000);
+  cpar[5]=nround(GRAClist[GC].zoom*10000);
   GRAdraw(GC,code,cpar,NULL);
 }
 
@@ -480,8 +498,12 @@ void GRAredraw2(struct objlist *obj,char *inst,int setredrawf,int redrawf,
   if (_getobj(obj,"oid",inst,&oid)) return;
   if ((snum=arraynum(sarray))==0) return;
   sdata=(char **)arraydata(sarray);
-  if ((_putobj(obj,"_list",inst,NULL))
-  || (_getobj(obj,"_GC",inst,&GC))) {
+  if ((_putobj(obj,"_list",inst,NULL)) || (_getobj(obj,"_GC",inst,&GC))) {
+    arrayfree2(sarray);
+    return;
+  }
+  if (((gobj=getobjlist(sdata[0],&gid,&gfield,NULL))==NULL)
+  || ((ginst=getobjinstoid(gobj,gid))==NULL)) {
     arrayfree2(sarray);
     return;
   }
@@ -493,11 +515,6 @@ void GRAredraw2(struct objlist *obj,char *inst,int setredrawf,int redrawf,
     GRAreopen(GC);
   } else {
   /* gra is already closed */
-    if (((gobj=getobjlist(sdata[0],&gid,&gfield,NULL))==NULL)
-    || ((ginst=getobjinstoid(gobj,gid))==NULL)) {
-      arrayfree2(sarray);
-      return;
-    }
     /* check consistency */
     if (_getobj(gobj,"_device",ginst,&device) || (device==NULL)
     || ((xobj=getobjlist(device,&xid,&xfield,NULL))==NULL)
@@ -652,7 +669,7 @@ int GRAcharwidth(unsigned int ch, char *font,int size)
 
   for (i=GRAClimit-1;i>=0;i--)
     if ((GRAopened(i)==i) && (GRAClist[i].charwidth!=-1)) break;
-  if (i==-1) return round(25.4/72000.0*size*600);
+  if (i==-1) return nround(25.4/72000.0*size*600);
   argv[0]=GRAClist[i].objname;
   argv[1]="_charwidth";
   argv[2]=GRAClist[i].local;
@@ -672,7 +689,7 @@ int GRAcharascent(char *font,int size)
 
   for (i=GRAClimit-1;i>=0;i--)
     if ((GRAopened(i)==i) && (GRAClist[i].charascent!=-1)) break;
-  if (i==-1) return round(25.4/72000.0*size*563);
+  if (i==-1) return nround(25.4/72000.0*size*563);
   argv[0]=GRAClist[i].objname;
   argv[1]="_charascent";
   argv[2]=GRAClist[i].local;
@@ -691,7 +708,7 @@ int GRAchardescent(char *font,int size)
 
   for (i=GRAClimit-1;i>=0;i--)
     if ((GRAopened(i)==i) && (GRAClist[i].chardescent!=-1)) break;
-  if (i==-1) return round(25.4/72000.0*size*250);
+  if (i==-1) return nround(25.4/72000.0*size*250);
   argv[0]=GRAClist[i].objname;
   argv[1]="_chardescent";
   argv[2]=GRAClist[i].local;
@@ -1609,6 +1626,9 @@ void GRAoutkanji(int GC,char *s)
   GRAdraw(GC,code,cpar,cstr);
 }
 
+char *GRAexpandobj(char **s);
+char *GRAexpandmath(char **s);
+
 char *GRAexpandobj(char **s)
 {
   struct objlist *obj;
@@ -1640,6 +1660,11 @@ char *GRAexpandobj(char **s)
       arg=nstrcat(arg,ret);
       memfree(ret);
       if (arg==NULL) goto errexit;
+    } else if (!quote && ((*s)[0]=='%') && ((*s)[1]=='[')) {
+      ret=GRAexpandmath(s);
+      arg=nstrcat(arg,ret);
+      memfree(ret);
+      if (str==NULL) goto errexit;
     } else if (!quote && ((*s)[0]=='\\')) {
       quote=TRUE;
       (*s)++;
@@ -1650,26 +1675,24 @@ char *GRAexpandobj(char **s)
     }
   }
   if ((*s)[0]!='}') goto errexit;
-  else {
-    (*s)++;
-    for (i=0;i<anum;i++) {
-      if (schkobjfield(obj,id[i],field,arg,&ret,TRUE,FALSE,FALSE)!=0) 
-        goto errexit;
-      for (j=0;ret[j]!='\0';j++) {
-        if ((ret[j]=='%') || (ret[j]=='\\')
-         || (ret[j]=='_') || (ret[j]=='^') || (ret[j]=='@')) {
-          if ((str=nstrccat(str,'\\'))==NULL) {
-            memfree(ret);
-            goto errexit;
-          }
-        }
-        if ((str=nstrccat(str,ret[j]))==NULL) {
+  (*s)++;
+  for (i=0;i<anum;i++) {
+    if (schkobjfield(obj,id[i],field,arg,&ret,TRUE,FALSE,FALSE)!=0)
+      goto errexit;
+    for (j=0;ret[j]!='\0';j++) {
+      if ((ret[j]=='%') || (ret[j]=='\\')
+       || (ret[j]=='_') || (ret[j]=='^') || (ret[j]=='@')) {
+        if ((str=nstrccat(str,'\\'))==NULL) {
           memfree(ret);
           goto errexit;
         }
       }
-      memfree(ret);
+      if ((str=nstrccat(str,ret[j]))==NULL) {
+        memfree(ret);
+        goto errexit;
+      }
     }
+    memfree(ret);
   }
   arraydel(&iarray);
   memfree(field);
@@ -1683,6 +1706,184 @@ errexit:
   return NULL;
 }
 
+char *GRAexpandmath(char **s)
+{
+  int j;
+  char *str,*ret;
+  int quote;
+  int rcode;
+  char *code;
+  double vd;
+  double memory[MEMORYNUM];
+  char memorystat[MEMORYNUM];
+
+  *s=*s+2;
+  str=NULL;
+  if ((str=nstrnew())==NULL) goto errexit;
+  if ((*s)==NULL) goto errexit;
+  quote=FALSE;
+  while (((*s)[0]!='\0') && (quote || ((*s)[0]!=']'))) {
+    if (!quote && ((*s)[0]=='%') && ((*s)[1]=='{')) {
+      ret=GRAexpandobj(s);
+      str=nstrcat(str,ret);
+      memfree(ret);
+      if (str==NULL) goto errexit;
+    } else if (!quote && ((*s)[0]=='%') && ((*s)[1]=='[')) {
+      ret=GRAexpandmath(s);
+      str=nstrcat(str,ret);
+      memfree(ret);
+      if (str==NULL) goto errexit;
+    } else if (!quote && ((*s)[0]=='\\')) {
+      quote=TRUE;
+      (*s)++;
+    } else {
+      if (quote) quote=FALSE;
+      if ((str=nstrccat(str,(*s)[0]))==NULL) goto errexit;
+      (*s)++;
+    }
+  }
+  if ((*s)[0]!=']') goto errexit;
+  (*s)++;
+  rcode=mathcode(str,&code,NULL,NULL,NULL,NULL,
+                 FALSE,FALSE,FALSE,FALSE,FALSE,
+                 FALSE,FALSE,FALSE,FALSE,FALSE,FALSE);
+  if (rcode!=MCNOERR) goto errexit;
+  for (j=0;j<MEMORYNUM;j++) {memory[j]=0;memorystat[j]=MNOERR;}
+  rcode=calculate(code,1,
+                  0,MNOERR,0,MNOERR,0,MNOERR,
+                  0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+                  NULL,NULL,
+                  memory,memorystat,
+                  NULL,NULL,
+                  NULL,NULL,
+                  NULL,NULL,NULL,
+                  NULL,NULL,NULL,0,NULL,NULL,NULL,0,&vd);
+  memfree(code);
+  if (rcode!=MNOERR) goto errexit;
+  memfree(str);
+  if ((str=memalloc(24))==NULL) goto errexit;
+  sprintf(str,"%.15e",vd);
+  return str;
+errexit:
+  memfree(str);
+  return NULL;
+}
+
+char *GRAexpandpf(char **s)
+{
+  int i,j,len,len1,len2,err;
+  char *str,*format,*format2,*s2,*ret,*ret2;
+  int quote;
+  int vi;
+  double vd;
+  char *buf;
+  char *endptr;
+
+  *s=*s+4;
+  str=NULL;
+  format=NULL;
+  if ((ret=nstrnew())==NULL) goto errexit;
+  if ((str=nstrnew())==NULL) goto errexit;
+  if ((*s)==NULL) goto errexit;
+  if ((format=getitok2(s,&len," \t}"))==NULL) goto errexit;
+  if (((*s)[0]!='\0') && ((*s)[0]!='}')) (*s)++;
+  while (((*s)[0]==' ') || ((*s)[0]=='\t')) (*s)++;
+  quote=FALSE;
+  while (((*s)[0]!='\0') && (quote || ((*s)[0]!='}'))) {
+    if (!quote && ((*s)[0]=='%') && ((*s)[1]=='{')) {
+      ret2=GRAexpandobj(s);
+      str=nstrcat(str,ret2);
+      memfree(ret2);
+      if (str==NULL) goto errexit;
+    } else if (!quote && ((*s)[0]=='%') && ((*s)[1]=='[')) {
+      ret2=GRAexpandmath(s);
+      str=nstrcat(str,ret2);
+      memfree(ret2);
+      if (str==NULL) goto errexit;
+    } else if (!quote && ((*s)[0]=='\\')) {
+      quote=TRUE;
+      (*s)++;
+    } else {
+      if (quote) quote=FALSE;
+      if ((str=nstrccat(str,(*s)[0]))==NULL) goto errexit;
+      (*s)++;
+    }
+  }
+  if ((*s)[0]!='}') goto errexit;
+  (*s)++;
+
+  if (format[0]=='%') {
+    for (i=1;(format[i]!='\0')
+         && (strchr("diouxXeEfgcs%",format[i])==NULL);i++);
+    if (format[i]!='\0') {
+      if ((format2=memalloc(i+2))==NULL) goto errexit;
+      strncpy(format2,format,i+1);
+      format2[i+1]='\0';
+      len1=len2=0;
+      err=FALSE;
+      s2=format2;
+      for (j=0;(s2[j]!='\0') && (!isdigit(s2[j]));j++);
+      if (isdigit(s2[j])) {
+        len1=strtol(s2+j,&endptr,10);
+        if (len1<0) len1=0;
+        if (endptr[0]=='.') {
+          s2=endptr;
+          for (j=0;(s2[j]!='\0') && (strchr(".*",s2[j])!=NULL);j++);
+          if (isdigit(s2[j])) {
+            len2=strtol(s2+j,&endptr,10);
+            if (len2<0) len2=0;
+          } else err=TRUE;
+        }
+      }
+      if (strchr(format2,'*')!=NULL) err=TRUE;
+      if (!err) {
+        len=len1+len2+256;
+        switch (format[i]) {
+        case 'd': case 'i': case 'o': case 'u': case 'x': case 'X':
+          vi=strtol(str,&endptr,10);
+          if ((buf=memalloc(len))!=NULL) {
+            sprintf(buf,format2,vi);
+            ret=nstrcat(ret,buf);
+            memfree(buf);
+          }
+          break;
+        case 'e': case 'E': case 'f': case 'g':
+          vd=strtod(str,&endptr);
+          if ((buf=memalloc(len))!=NULL) {
+            sprintf(buf,format2,vd);
+            ret=nstrcat(ret,buf);
+            memfree(buf);
+          }
+          break;
+        case 's':
+          if ((buf=memalloc(len+strlen(str)))!=NULL) {
+            sprintf(buf,format2,str);
+            ret=nstrcat(ret,buf);
+            memfree(buf);
+          }
+          break;
+        case 'c':
+          if ((buf=memalloc(len+strlen(str)))!=NULL) {
+            sprintf(buf,format2,str[0]);
+            ret=nstrcat(ret,buf);
+            memfree(buf);
+          }
+        }
+      }
+      memfree(format2);
+      if (ret==NULL) goto errexit;
+    }
+  }
+
+  memfree(str);
+  memfree(format);
+  return ret;
+errexit:
+  memfree(str);
+  memfree(format);
+  return NULL;
+}
+
 char *GRAexpandtext(char *s)
 {
   int i,j,len;
@@ -1711,6 +1912,12 @@ char *GRAexpandtext(char *s)
       } else if (s[j+1]=='.') {
         if ((str=nstrccat(str,' '))==NULL) return NULL;
         j+=2;
+      } else if (s[j+1]=='-') {
+        if ((str=nstrccat(str,'\\'))==NULL) return NULL;
+        if ((str=nstrccat(str,'x'))==NULL) return NULL;
+        if ((str=nstrccat(str,'A'))==NULL) return NULL;
+        if ((str=nstrccat(str,'D'))==NULL) return NULL;
+        j+=2;
       } else if ((s[j+1]=='x') && isxdigit(s[j+2]) && isxdigit(s[j+3])) {
         if ((str=nstrccat(str,'\\'))==NULL) return NULL;
         if ((str=nstrccat(str,'x'))==NULL) return NULL;
@@ -1730,6 +1937,18 @@ char *GRAexpandtext(char *s)
       if ((str=nstrcat(str,snew))==NULL) return NULL;
       memfree(snew);
       j=(s2-s);
+    } else if ((s[j]=='%') && (s[j+1]=='[')) {
+      s2=s+j;
+      snew=GRAexpandmath(&s2);
+      if ((str=nstrcat(str,snew))==NULL) return NULL;
+      memfree(snew);
+      j=(s2-s);
+    } else if ((s[j]=='%') && (s[j+1]=='p') && (s[j+2]=='f') && (s[j+3]=='{')) {
+      s2=s+j;
+      snew=GRAexpandpf(&s2);
+      if ((str=nstrcat(str,snew))==NULL) return NULL;
+      memfree(snew);
+      j=(s2-s);
     } else if (j<len) {
       if ((str=nstrccat(str,s[j]))==NULL) return NULL;
       j++;
@@ -2118,7 +2337,7 @@ void GRAtextextent(char *s,char *font,char *jfont,
         w=0;
         for (i=0;i<strlen(str);i+=2) {
           w+=GRAcharwidth((((unsigned char)str[i+1])<<8)+(unsigned char)str[i],
-                           jfont2,size2)+round(space2/72.0*25.4);
+                           jfont2,size2)+nround(space2/72.0*25.4);
         }
         h=GRAcharascent(jfont2,size2);
         d=GRAchardescent(jfont2,size2);
@@ -2132,11 +2351,11 @@ void GRAtextextent(char *s,char *font,char *jfont,
             else ch=ch*16+str[i+3]-'0';
             str[i]=ch;
             w+=GRAcharwidth((unsigned char)str[i],font2,size2)
-              +round(space2/72.0*25.4);
+              +nround(space2/72.0*25.4);
             i+=3;
           } else {
             w+=GRAcharwidth((unsigned char)str[i],font2,size2)
-              +round(space2/72.0*25.4);
+              +nround(space2/72.0*25.4);
           }
         }
         h=GRAcharascent(font2,size2);
@@ -2328,14 +2547,14 @@ void GRAtextextentraw(char *s,char *font,char *jfont,
         w=0;
         for (i=0;i<strlen(str);i+=2) {
           w+=GRAcharwidth((((unsigned char)str[i+1])<<8)+(unsigned char)str[i],
-                           jfont,size)+round(space/72.0*25.4);
+                           jfont,size)+nround(space/72.0*25.4);
         }
         h=GRAcharascent(jfont,size);
       } else {
         w=0;
         for (i=0;i<strlen(str);i++)
           w+=GRAcharwidth((unsigned char)str[i],font,size)
-            +round(space/72.0*25.4);
+            +nround(space/72.0*25.4);
         h=GRAcharascent(font,size);
       }
       if (x0<*gx0) *gx0=x0;
@@ -2613,7 +2832,7 @@ int GRAinputold(int GC,char *s,int leftm,int topm,int rate,int greek)
     break;
   case 'I':
     cpar[0]=5;
-    cpar[5]=round(cpar[3]/2.1);
+    cpar[5]=nround(cpar[3]/2.1);
     cpar[3]=21000;
     cpar[4]=29700;
     GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
@@ -2889,8 +3108,8 @@ void GRAcurvefirst(int GC,int num,int *dashlist,
   GRAClist[GC].x0=x0;
   GRAClist[GC].y0=y0;
   if (GRAClist[GC].gtransf==NULL) {
-    gx0=round(x0);
-    gy0=round(y0);
+    gx0=nround(x0);
+    gy0=nround(y0);
   } else GRAClist[GC].gtransf(x0,y0,&gx0,&gy0,local);
   GRAmoveto(GC,gx0,gy0);
 }
@@ -2928,10 +3147,10 @@ void GRAdashlinetod(int GC,double x,double y)
   if ((GRAClist[GC].gclipf==NULL)
    || (GRAClist[GC].gclipf(&x1,&y1,&x2,&y2,GRAClist[GC].gflocal)==0)) {
     if (GRAClist[GC].gtransf==NULL) {
-      gx1=round(x1);
-      gy1=round(y1);
-      gx2=round(x2);
-      gy2=round(y2);
+      gx1=nround(x1);
+      gy1=nround(y1);
+      gx2=nround(x2);
+      gy2=nround(y2);
     } else {
       GRAClist[GC].gtransf(x1,y1,&gx1,&gy1,GRAClist[GC].gflocal);
       GRAClist[GC].gtransf(x2,y2,&gx2,&gy2,GRAClist[GC].gflocal);
@@ -2947,8 +3166,8 @@ void GRAdashlinetod(int GC,double x,double y)
       >((GRAClist[GC].gdashlist)[GRAClist[GC].gdashi]-GRAClist[GC].gdashlen)) {
         dd=(len-len2+(GRAClist[GC].gdashlist)[GRAClist[GC].gdashi]
            -GRAClist[GC].gdashlen)/len;
-        gx=gx1+round(dx*dd);
-        gy=gy1+round(dy*dd);
+        gx=gx1+nround(dx*dd);
+        gy=gy1+nround(dy*dd);
         if (GRAClist[GC].gdotf) GRAlineto(GC,gx,gy);
         else GRAmoveto(GC,gx,gy);
         GRAClist[GC].gdotf=GRAClist[GC].gdotf ? FALSE : TRUE;
@@ -3004,10 +3223,10 @@ void GRAcmatchtod(double x,double y,struct cmatchtype *data)
   y2=y;
   if (data->bbox) {
     if (data->gtransf==NULL) {
-      gx1=round(x1);
-      gy1=round(y1);
-      gx2=round(x2);
-      gy2=round(y2);
+      gx1=nround(x1);
+      gy1=nround(y1);
+      gx2=nround(x2);
+      gy2=nround(y2);
     } else {
       data->gtransf(x1,y1,&gx1,&gy1,data->gflocal);
       data->gtransf(x2,y2,&gx2,&gy2,data->gflocal);
@@ -3025,10 +3244,10 @@ void GRAcmatchtod(double x,double y,struct cmatchtype *data)
     if ((data->gclipf==NULL)
      || (data->gclipf(&x1,&y1,&x2,&y2,data->gflocal)==0)) {
       if (data->gtransf==NULL) {
-        gx1=round(x1);
-        gy1=round(y1);
-        gx2=round(x2);
-        gy2=round(y2);
+        gx1=nround(x1);
+        gy1=nround(y1);
+        gx2=nround(x2);
+        gy2=nround(y2);
       } else {
         data->gtransf(x1,y1,&gx1,&gy1,data->gflocal);
         data->gtransf(x2,y2,&gx2,&gy2,data->gflocal);
