@@ -1,21 +1,38 @@
-/**
- *
- * $Id: ioutil.c,v 1.2 1999/04/15 12:15:27 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: ioutil.c,v 1.5 2003/02/16 12:42:14 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: ioutil.c,v $
+ * Revision 1.5  2003/02/16 12:42:14  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.4  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.3  2001/03/23 12:15:31  isizaka
+ * for 6.3.13
+ *
  * Revision 1.2  1999/04/15 12:15:27  isizaka
  * for release 6.03.01
  *
@@ -52,11 +69,12 @@
 #ifndef WINDOWS
 void changefilename(char *name)
 {
-  int i,j;
+  int i;
 
   if (name==NULL) return;
   for (i=0;name[i]!='\0';i++)
     if ((name[i]=='\\') && (!niskanji2(name,i))) name[i]=DIRSEP;
+/*
   if ((name[0]==DIRSEP) && (name[1]==DIRSEP)) {
     for (i=2;(name[i]!='\0') && (name[i]!=DIRSEP);i++);
     if (name[i]==DIRSEP) i++;
@@ -64,17 +82,19 @@ void changefilename(char *name)
   j=i;
   if (isalpha(name[j]) && (name[j+1]==':'))
     for (i=j+2;name[i]!='\0';i++) name[j++]=name[i];
+*/
 }
 #else
 void changefilename(char *name)
 {
   int i;
-  char disk[4];
-  DWORD csensitive;
+/*  char disk[4];
+  DWORD csensitive; */
 
   if (name==NULL) return;
   for (i=0;name[i]!='\0';i++)
     if ((name[i]=='\\') && (!niskanji2(name,i))) name[i]=DIRSEP;
+/*
   if ((name[0]==DIRSEP) && (name[1]==DIRSEP)) return;
   if (isalpha(name[0]) && (name[1]==':')) {
     name[0]=tolower(name[0]);
@@ -92,6 +112,7 @@ void changefilename(char *name)
       else name[i]=tolower(name[i]);
     }
   }
+*/
 }
 
 void unchangefilename(char *name)
@@ -291,7 +312,7 @@ int findfilename(char *dir,char *sep,char *file)
 char *ngetcwd()
 {
   char *buf,*s;
-  int size;
+  size_t size;
 
   size=0;
   buf=NULL;
@@ -300,8 +321,8 @@ char *ngetcwd()
     size+=256;
     if ((buf=memalloc(size))==NULL) return NULL;
     s=getcwd(buf,size);
-  } while (errno==ERANGE);
-  if (s==NULL) {
+  } while ((s==NULL) && (size<=10240));
+  if (size>10240) {
     memfree(buf);
     return NULL;
   }
@@ -676,6 +697,16 @@ int fgetnline(FILE *fp,char *buf,int len)
   }
 }
 
+int nfgetc(FILE *fp)
+{
+  int ch;
+
+  do {
+    ch=fgetc(fp);
+  } while (ch=='\r');
+  return ch;
+}
+
 FILE *nfopen(char *filename, const char *mode)
 {
   FILE *fp;
@@ -705,6 +736,21 @@ void nclose(int fd)
   close(fd);
 }
 
+void nlseek(int fd,long offset,int fromwhere)
+{
+  lseek(fd,offset,fromwhere);
+}
+
+int nread(int fd,char *buf,unsigned len)
+{
+  return read(fd,buf,len);
+}
+
+int nwrite(int fd,char *buf,unsigned len)
+{
+  return write(fd,buf,len);
+}
+
 int nredirect(int fd,int newfd)
 {
   int savefd;
@@ -721,21 +767,6 @@ void nredirect2(int fd,int savefd)
   close(savefd);
 }
 
-void nlseek(int fd,long offset,int fromwhere)
-{
-  lseek(fd,offset,fromwhere);
-}
-
-int nread(int fd,char *buf,unsigned len)
-{
-  return read(fd,buf,len);
-}
-
-int nwrite(int fd,char *buf,unsigned len)
-{
-  return write(fd,buf,len);
-}
-
 int stdinfd(void)
 {
   return 0;
@@ -796,50 +827,6 @@ void nclose(HANDLE fd)
   CloseHandle(fd);
 }
 
-HANDLE nredirect(int fd,HANDLE newfd)
-{
-  HANDLE savefd;
-
-  switch (fd) {
-  case 0:
-    savefd=GetStdHandle(STD_INPUT_HANDLE);
-    SetStdHandle(STD_INPUT_HANDLE,newfd);
-    break;
-  case 1:
-    savefd=GetStdHandle(STD_OUTPUT_HANDLE);
-    SetStdHandle(STD_OUTPUT_HANDLE,newfd);
-    break;
-  case 2:
-    savefd=GetStdHandle(STD_ERROR_HANDLE);
-    SetStdHandle(STD_ERROR_HANDLE,newfd);
-    break;
-  }
-  return savefd;
-}
-
-void nredirect2(int fd,HANDLE savefd)
-{
-  HANDLE oldfd;
-
-  switch (fd) {
-  case 0:
-    oldfd=GetStdHandle(STD_INPUT_HANDLE);
-    CloseHandle(oldfd);
-    SetStdHandle(STD_INPUT_HANDLE,savefd);
-    break;
-  case 1:
-    oldfd=GetStdHandle(STD_OUTPUT_HANDLE);
-    CloseHandle(oldfd);
-    SetStdHandle(STD_OUTPUT_HANDLE,savefd);
-    break;
-  case 2:
-    oldfd=GetStdHandle(STD_ERROR_HANDLE);
-    CloseHandle(oldfd);
-    SetStdHandle(STD_ERROR_HANDLE,savefd);
-    break;
-  }
-}
-
 void nlseek(HANDLE fd,long offset,int fromwhere)
 {
   LONG DistanceToMove;
@@ -888,6 +875,50 @@ int nwrite(HANDLE fd,char *buf,unsigned len)
   return len2-num;
 }
 
+HANDLE nredirect(int fd,HANDLE newfd)
+{
+  HANDLE savefd;
+
+  switch (fd) {
+  case 0:
+    savefd=GetStdHandle(STD_INPUT_HANDLE);
+    SetStdHandle(STD_INPUT_HANDLE,newfd);
+    break;
+  case 1:
+    savefd=GetStdHandle(STD_OUTPUT_HANDLE);
+    SetStdHandle(STD_OUTPUT_HANDLE,newfd);
+    break;
+  case 2:
+    savefd=GetStdHandle(STD_ERROR_HANDLE);
+    SetStdHandle(STD_ERROR_HANDLE,newfd);
+    break;
+  }
+  return savefd;
+}
+
+void nredirect2(int fd,HANDLE savefd)
+{
+  HANDLE oldfd;
+
+  switch (fd) {
+  case 0:
+    oldfd=GetStdHandle(STD_INPUT_HANDLE);
+    CloseHandle(oldfd);
+    SetStdHandle(STD_INPUT_HANDLE,savefd);
+    break;
+  case 1:
+    oldfd=GetStdHandle(STD_OUTPUT_HANDLE);
+    CloseHandle(oldfd);
+    SetStdHandle(STD_OUTPUT_HANDLE,savefd);
+    break;
+  case 2:
+    oldfd=GetStdHandle(STD_ERROR_HANDLE);
+    CloseHandle(oldfd);
+    SetStdHandle(STD_ERROR_HANDLE,savefd);
+    break;
+  }
+}
+
 HANDLE stdinfd(void)
 {
   return GetStdHandle(STD_INPUT_HANDLE);
