@@ -1,21 +1,38 @@
-/**
- *
- * $Id: mathcode.c,v 1.3 1999/05/31 10:33:41 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: mathcode.c,v 1.6 2002/07/06 08:51:42 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: mathcode.c,v $
+ * Revision 1.6  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.5  2001/03/23 12:15:31  isizaka
+ * for 6.3.13
+ *
+ * Revision 1.4  2001/01/24 12:24:48  isizaka
+ * bug fix: memorystat[]
+ *
  * Revision 1.3  1999/05/31 10:33:41  isizaka
  * for release 6.03.03
  *
@@ -40,7 +57,7 @@
 #define TRUE 1
 #define FALSE 0
 
-#define OPENUM 61
+#define OPENUM 65
 
 char *matherrorchar[10]={
  "noerr",
@@ -113,16 +130,31 @@ struct {
   {"RM(",   '}',1},
   {"SUM(",  '[',1},
   {"DIF(",  ']',1},
-  {"F(", (char )0xa0,3},
-  {"G(", (char )0xa1,3},
-  {"H(", (char )0xa2,3},
-  {"IF(",   '?',3},
-  {"FOR(",  ':',5},
-  {"COLOR(",  '"',2},
-  {"MARKSIZE(",  '\'',1},
-  {"QINV(", (char )0xc0,1},
+  {"MJD(",  '<',3},
+  {"QINV(", '>',1},
+  {"F(",(char )0xa0,3},
+  {"G(",(char )0xa1,3},
+  {"H(",(char )0xa2,3},
+  {"IF(",'?',3},
+  {"FOR(",':',5},
+  {"COLOR(",'"',2},
+  {"RGB(",(char )0xc0,3},
+  {"HSB(",(char )0xc1,3},
+  {"MARKSIZE(",'\'',1},
+  {"MARKTYPE(",'$',1},
 };
 
+/*
+used character code:
+
+=XYZ#|+-/*\^()!,"@
+8x
+9x
+b0 b1 b2 b3
+d0 d1 d2 d3 d4 d5 d6 d7 d8 d9 da db dc dd
+
+*/
+
 int mathcode(char *str,char **code,
     struct narray *needdata,struct narray *needfile,
     int *maxdim,int *twopass,
@@ -167,14 +199,32 @@ int mathcode(char *str,char **code,
       for (j=0;j<sizeof(x);j++)
         if (arrayadd(ncode,((char *)&x)+j)==NULL) goto errexit;
       po+=len;
-    } else if ((s[po]=='%') && isdigit(s[po+1]) && isdigit(s[po+2])) {
+    } else if ((s[po]=='%') && isdigit(s[po+1])) {
       if (!column) {
         rcode=MCILLEGAL;
         goto errexit;
       }
-      if (arrayadd(ncode,s+po+1)==NULL) goto errexit;
-      if (arrayadd(ncode,s+po+2)==NULL) goto errexit;
-      dim=(s[po+1]-'0')*10+(s[po+2]-'0');
+      if (isdigit(s[po+2]) && isdigit(s[po+3])) {
+        if (arrayadd(ncode,s+po+1)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+2)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+3)==NULL) goto errexit;
+        dim=(s[po+1]-'0')*100+(s[po+2]-'0')*10+(s[po+3]-'0');
+        po+=4;
+      } else if (isdigit(s[po+2])) {
+        ch='0';
+        if (arrayadd(ncode,&ch)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+1)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+2)==NULL) goto errexit;
+        dim=(s[po+1]-'0')*10+(s[po+2]-'0');
+        po+=3;
+      } else {
+        ch='0';
+        if (arrayadd(ncode,&ch)==NULL) goto errexit;
+        if (arrayadd(ncode,&ch)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+1)==NULL) goto errexit;
+        dim=(s[po+1]-'0');
+        po+=2;
+      }
       if (*maxdim<dim) *maxdim=dim;
       if (needdata!=NULL) {
         ndata=arraydata(needdata);
@@ -182,7 +232,6 @@ int mathcode(char *str,char **code,
         for (i=0;i<nn;i++) if (ndata[i]==dim) break;
         if (i==nn) arrayadd(needdata,&dim);
       }
-      po+=3;
     } else if ((s[po]=='%') && (s[po+1]=='D')) {
       if (!file) {
         rcode=MCILLEGAL;
@@ -192,8 +241,7 @@ int mathcode(char *str,char **code,
       if (arrayadd(ncode,&ch)==NULL) goto errexit;
       po+=2;
     } else if ((s[po]=='%') && (s[po+1]=='F')
-    && isdigit(s[po+2]) && isdigit(s[po+3]) && isdigit(s[po+4]) 
-    && isdigit(s[po+5])) {
+      && isdigit(s[po+2]) && isdigit(s[po+3]) && isdigit(s[po+4])) {
       if (!file || !column) {
         rcode=MCILLEGAL;
         goto errexit;
@@ -202,11 +250,29 @@ int mathcode(char *str,char **code,
       if (arrayadd(ncode,&ch)==NULL) goto errexit;
       if (arrayadd(ncode,s+po+2)==NULL) goto errexit;
       if (arrayadd(ncode,s+po+3)==NULL) goto errexit;
-      if (arrayadd(ncode,s+po+4)==NULL) goto errexit;
-      if (arrayadd(ncode,s+po+5)==NULL) goto errexit;
       id=(s[po+2]-'0')*10+(s[po+3]-'0');
-      dim=(s[po+4]-'0')*10+(s[po+5]-'0');
-      maxdim2=id*100+dim;
+      if (isdigit(s[po+5]) && isdigit(s[po+6])) {
+        if (arrayadd(ncode,s+po+4)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+5)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+6)==NULL) goto errexit;
+        dim=(s[po+4]-'0')*100+(s[po+5]-'0')*10+(s[po+6]-'0');
+        po+=7;
+      } else if (isdigit(s[po+5])) {
+        ch='0';
+        if (arrayadd(ncode,&ch)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+4)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+5)==NULL) goto errexit;
+        dim=(s[po+4]-'0')*10+(s[po+5]-'0');
+        po+=6;
+      } else {
+        ch='0';
+        if (arrayadd(ncode,&ch)==NULL) goto errexit;
+        if (arrayadd(ncode,&ch)==NULL) goto errexit;
+        if (arrayadd(ncode,s+po+4)==NULL) goto errexit;
+        dim=(s[po+4]-'0');
+        po+=5;
+      }
+      maxdim2=id*1000+dim;
       if (needfile!=NULL) {
         ndata=arraydata(needfile);
         nn=arraynum(needfile);
@@ -219,7 +285,6 @@ int mathcode(char *str,char **code,
         }
         if (i==nn) arrayadd(needfile,&maxdim2);
       }
-      po+=6;
     } else if (strchr("+-*/\\^()!,",s[po])!=NULL) {
       if (arrayadd(ncode,s+po)==NULL) goto errexit;
       po++;
@@ -256,18 +321,133 @@ int mathcode(char *str,char **code,
           ch=(char )0xb2;
           if (arrayadd(ncode,&ch)==NULL) goto errexit;
           po+=5;
-        } else if (((strncmp(s+po,"MIN",3)==0) || (strncmp(s+po,"MAX",3)==0))
-                 && ((s[po+3]=='X') || (s[po+3]=='Y') || s[po+3]=='Z')) {
+        } else if (strncmp(s+po,"NUM",3)==0) {
           if (!minmax) {
             rcode=MCILLEGAL;
             goto errexit;
-     	    }
-          if (s[po+1]=='I') ch='<';
-          else ch='>';
+          }
+          ch=(char )0xd0;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=3;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"MINX",4)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd1;
           if (arrayadd(ncode,&ch)==NULL) goto errexit;
-          if (arrayadd(ncode,s+po+3)==NULL) goto errexit;
           po+=4;
           *twopass=TRUE;
+        } else if (strncmp(s+po,"MINY",4)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd2;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=4;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"MAXX",4)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd3;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=4;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"MAXY",4)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd4;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=4;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"SUMXX",5)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd7;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=5;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"SUMYY",5)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd8;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=5;
+          *twopass=TRUE;
+        } else if ((strncmp(s+po,"SUMXY",5)==0)
+                || (strncmp(s+po,"SUMYX",5)==0)) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd9;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=5;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"SUMX",4)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd5;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=4;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"SUMY",4)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xd6;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=4;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"AVX",3)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xda;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=3;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"AVY",3)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xdb;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=3;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"SGX",3)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xdc;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=3;
+          *twopass=TRUE;
+        } else if (strncmp(s+po,"SGY",3)==0) {
+          if (!minmax) {
+            rcode=MCILLEGAL;
+            goto errexit;
+          }
+          ch=(char )0xdd;
+          if (arrayadd(ncode,&ch)==NULL) goto errexit;
+          po+=3;
+          *twopass=TRUE;
         } else if ((strncmp(s+po,"EULER",5)==0) || (s[po]=='E')
                 || (strncmp(s+po,"PI",2)==0)) {
           ch='#';
@@ -310,11 +490,14 @@ int mathcode(char *str,char **code,
           rcode=MCILLEGAL;
           goto errexit;
         }
-        if ((!color) && (opetable[i].code=='"')) {
+        if ((!color) 
+        && ((opetable[i].code=='"')
+        || (opetable[i].code==(char )0xc0) || (opetable[i].code==(char )0xc1))) {
           rcode=MCILLEGAL;
           goto errexit;
-	    }
-        if ((!marksize) && (opetable[i].code=='\'')) {
+        }
+        if ((!marksize) &&
+            ((opetable[i].code=='\'') || (opetable[i].code=='$'))) {
           rcode=MCILLEGAL;
           goto errexit;
         }
@@ -329,7 +512,7 @@ int mathcode(char *str,char **code,
           if (integpo>=10) {
             rcode=MCNEST;
             goto errexit;
-	      }
+          }
           ch=(unsigned char)0x80+(unsigned char)integpo;
           if (arrayadd(ncode,&ch)==NULL) goto errexit;
           integpo++;
@@ -375,7 +558,7 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
      double memory[],char memorystat[],
      double sumdata[],char sumstat[],
      double difdata[],char difstat[],
-     int color[],int *marksize,
+     int color[],int *marksize,int *marktype,
      char *ufcodef,char *ufcodeg,char *ufcodeh)
 {
   double x;
@@ -403,7 +586,7 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
       status=MNOERR;
       for (i=1;i<=argnum;i++)
         if (numstat[*numpo-i+1]>status) status=numstat[*numpo-i+1];
-      if (status!=0) numstat[*numpo-argnum+1]=status;
+      if ((opbuf[*oppo]!='{') && (status!=0)) numstat[*numpo-argnum+1]=status;
       else {
           switch (opbuf[*oppo]) {
           case '!': 
@@ -596,9 +779,6 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
           case 'e':
             if (erfc1(numbuf[*numpo],&(numbuf[*numpo]))) numstat[*numpo]=MERR;
             break;
-          case (char )0xc0:
-            if (qinv1(numbuf[*numpo],&(numbuf[*numpo]))) numstat[*numpo]=MERR;
-            break;
           case 'f':
             if (exp1(numbuf[*numpo],&(numbuf[*numpo]))) numstat[*numpo]=MERR;
             break;
@@ -702,13 +882,19 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
               numstat[*numpo-1]=MERR;
             else {
               memory[(int )numbuf[*numpo-1]]=numbuf[*numpo];
+              memorystat[(int )numbuf[*numpo-1]]=numstat[*numpo];
               numbuf[*numpo-1]=numbuf[*numpo];
+              numstat[*numpo-1]=numstat[*numpo];
             }
             break;
           case '}':
             if ((numbuf[*numpo]-(int )numbuf[*numpo]!=0)
-            || (numbuf[*numpo]<0) || (numbuf[*numpo]>(MEMORYNUM-1))) numstat[*numpo]=MERR;
-            else numbuf[*numpo]=memory[(int )numbuf[*numpo]];
+            || (numbuf[*numpo]<0) || (numbuf[*numpo]>(MEMORYNUM-1)))
+              numstat[*numpo]=MERR;
+            else {
+              numstat[*numpo]=memorystat[(int )numbuf[*numpo]];
+              numbuf[*numpo]=memory[(int )numbuf[*numpo]];
+            }
             break;
           case '"':
             if ((numbuf[*numpo-1]-(int )numbuf[*numpo-1]!=0)
@@ -718,18 +904,50 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
               if (numbuf[*numpo]>255) numbuf[*numpo]=255;
               else if (numbuf[*numpo]<0) numbuf[*numpo]=0;
               if (((int )numbuf[*numpo-1])!=3)
-                color[(int )numbuf[*numpo-1]]=round(numbuf[*numpo]);
+                color[(int )numbuf[*numpo-1]]=nround(numbuf[*numpo]);
               else {
-                color[0]=round(numbuf[*numpo]);
-                color[1]=round(numbuf[*numpo]);
-                color[2]=round(numbuf[*numpo]);
+                color[0]=nround(numbuf[*numpo]);
+                color[1]=nround(numbuf[*numpo]);
+                color[2]=nround(numbuf[*numpo]);
               }
-              numbuf[*numpo-1]=numbuf[*numpo];
             }
+            numbuf[*numpo-1]=numbuf[*numpo];
+            break;
+          case (char )0xc0:
+            while (numbuf[*numpo-2]<0) numbuf[*numpo-2]+=1;
+            while (numbuf[*numpo-1]<0) numbuf[*numpo-1]+=1;
+            while (numbuf[*numpo]<0) numbuf[*numpo]+=1;
+            while (numbuf[*numpo-2]>1) numbuf[*numpo-2]-=1;
+            while (numbuf[*numpo-1]>1) numbuf[*numpo-1]-=1;
+            while (numbuf[*numpo]>1) numbuf[*numpo]-=1;
+            color[0]=nround(numbuf[*numpo-2]*255);
+            color[1]=nround(numbuf[*numpo-1]*255);
+            color[2]=nround(numbuf[*numpo]*255);
+            break;
+          case (char )0xc1:
+            while (numbuf[*numpo-2]<0) numbuf[*numpo-2]+=1;
+            while (numbuf[*numpo-1]<0) numbuf[*numpo-1]+=1;
+            while (numbuf[*numpo]<0) numbuf[*numpo]+=1;
+            while (numbuf[*numpo-2]>1) numbuf[*numpo-2]-=1;
+            while (numbuf[*numpo-1]>1) numbuf[*numpo-1]-=1;
+            while (numbuf[*numpo]>1) numbuf[*numpo]-=1;
+            HSB2RGB(numbuf[*numpo-2],numbuf[*numpo-1],numbuf[*numpo],
+                    &(color[0]),&(color[1]),&(color[2]));
             break;
           case '\'':
             if (numbuf[*numpo]<0) numstat[*numpo]=MERR;
-            else *marksize=round(numbuf[*numpo]);
+            else *marksize=nround(numbuf[*numpo]);
+            break;
+          case '$':
+            if (numbuf[*numpo]<0) numstat[*numpo]=MERR;
+            else *marktype=nround(numbuf[*numpo]);
+            break;
+          case '<':
+            numbuf[*numpo-2]
+              =mjd(numbuf[*numpo-2],numbuf[*numpo-1],numbuf[*numpo]);
+            break;
+          case '>':
+            if (qinv1(numbuf[*numpo],&(numbuf[*numpo]))) numstat[*numpo]=MERR;
             break;
           case (char )0x80: case (char )0x81:
           case (char )0x82: case (char )0x83:
@@ -773,17 +991,12 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
                             numbuf[*numpo-2],numstat[*numpo-2],
                             numbuf[*numpo-1],numstat[*numpo-1],
                             numbuf[*numpo],numstat[*numpo],
-                            0,0,
-                            0,0,
-                            0,0,
-                            0,0,
-                            0,0,
-                            0,0,
+                            0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                             NULL,NULL,
                             umemory,umemorystat,
                             NULL,NULL,
                             NULL,NULL,
-                            NULL,NULL,
+                            NULL,NULL,NULL,
                             ufcodef,ufcodeg,ufcodeh,
                             0,NULL,NULL,NULL,0,
                             &(numbuf[*numpo-2]));
@@ -805,13 +1018,13 @@ int calculate(char *code,
               double maxx,char maxxstat,
               double miny,char minystat,
               double maxy,char maxystat,
-              double minz,char minzstat,
-              double maxz,char maxzstat,
+              int num,
+              double sumx,double sumy,double sumxx,double sumyy,double sumxy,
               double *data,char *datastat,
               double *memory,char *memorystat,
               double *sumdata,char *sumstat,
               double *difdata,char *difstat,
-              int *color,int *marksize,
+              int *color,int *marksize,int *marktype,
               char *ufcodef,char *ufcodeg,char *ufcodeh,
               int fnum,int *needfile,double *fdata,char *fdatastat,int file,
               double *value)
@@ -941,10 +1154,10 @@ int calculate(char *code,
           if (numeric) return MSERR;
           numpo++;
           if (numpo>=20) return MSERR;
-          dim=(code[po]-'0')*10+(code[po+1]-'0');
+          dim=(code[po]-'0')*100+(code[po+1]-'0')*10+(code[po+2]-'0');
           numbuf[numpo]=data[dim];
           numstat[numpo]=datastat[dim];
-          po+=2;
+          po+=3;
           numeric=TRUE;
           break;
         case '|':
@@ -952,8 +1165,8 @@ int calculate(char *code,
           numpo++;
           if (numpo>=20) return MSERR;
           id=(code[po+1]-'0')*10+(code[po+2]-'0');
-          dim=(code[po+3]-'0')*10+(code[po+4]-'0');
-          maxdim2=id*100+dim;
+          dim=(code[po+3]-'0')*100+(code[po+4]-'0')*10+(code[po+5]-'0');
+          maxdim2=id*1000+dim;
           for (i=0;i<fnum;i++)
             if (needfile[i]==maxdim2) break;
           if (i==fnum) {
@@ -963,49 +1176,145 @@ int calculate(char *code,
             numbuf[numpo]=fdata[i];
             numstat[numpo]=fdatastat[i];
           }
-          po+=5;
+          po+=6;
           numeric=TRUE;
           break;
-        case '<':
+        case (char )0xd0:
           if (numeric) return MSERR;
           numpo++;
           if (numpo>=20) return MSERR;
-          switch (code[po+1]) {
-          case 'X':
-            numbuf[numpo]=minx;
-            numstat[numpo]=minxstat;
-            break;
-          case 'Y':
-            numbuf[numpo]=miny;
-            numstat[numpo]=minystat;
-            break;
-          case 'Z':
-            numbuf[numpo]=minz;
-            numstat[numpo]=minzstat;
-            break;
-          }
-          po+=2;
+          numbuf[numpo]=num;
+          numstat[numpo]=MNOERR;
+          po++;
           numeric=TRUE;
           break;
-        case '>':
+        case (char )0xd1:
           if (numeric) return MSERR;
           numpo++;
           if (numpo>=20) return MSERR;
-          switch (code[po+1]) {
-          case 'X':
-            numbuf[numpo]=maxx;
-            numstat[numpo]=maxxstat;
-            break;
-          case 'Y':
-            numbuf[numpo]=maxy;
-            numstat[numpo]=maxystat;
-            break;
-          case 'Z':
-            numbuf[numpo]=maxz;
-            numstat[numpo]=maxzstat;
-            break;
+          numbuf[numpo]=minx;
+          numstat[numpo]=minxstat;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd2:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=miny;
+          numstat[numpo]=minystat;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd3:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=maxx;
+          numstat[numpo]=maxxstat;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd4:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=maxy;
+          numstat[numpo]=maxystat;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd5:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=sumx;
+          numstat[numpo]=MNOERR;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd6:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=sumy;
+          numstat[numpo]=MNOERR;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd7:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=sumxx;
+          numstat[numpo]=MNOERR;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd8:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=sumyy;
+          numstat[numpo]=MNOERR;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xd9:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          numbuf[numpo]=sumxy;
+          numstat[numpo]=MNOERR;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xda:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          if (num==0) numstat[numpo]=MERR;
+          else {
+            numbuf[numpo]=sumx/num;
+            numstat[numpo]=MNOERR;
           }
-          po+=2;
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xdb:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          if (num==0) numstat[numpo]=MERR;
+          else {
+            numbuf[numpo]=sumy/num;
+            numstat[numpo]=MNOERR;
+          }
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xdc:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          if (num==0) numstat[numpo]=MERR;
+          else {
+            numbuf[numpo]=sqrt(sumxx/num-(sumx/num)*(sumx/num));
+            numstat[numpo]=MNOERR;
+          }
+          po++;
+          numeric=TRUE;
+          break;
+        case (char )0xdd:
+          if (numeric) return MSERR;
+          numpo++;
+          if (numpo>=20) return MSERR;
+          if (num==0) numstat[numpo]=MERR;
+          else {
+            numbuf[numpo]=sqrt(sumyy/num-(sumy/num)*(sumy/num));
+            numstat[numpo]=MNOERR;
+          }
+          po++;
           numeric=TRUE;
           break;
         case '&':
@@ -1025,7 +1334,8 @@ int calculate(char *code,
         case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
         case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
         case 'v': case 'w': case 'x': case 'y': case 'z':
-        case '{': case '}': case '"': case '\'': case '?': case ':':
+        case '{': case '}': case '"': case '\'': case '$': case '?': case ':':
+        case '<': case '>':
         case (char )0x80: case (char )0x81: 
         case (char )0x82: case (char )0x83: 
         case (char )0x84: case (char )0x85:
@@ -1036,7 +1346,8 @@ int calculate(char *code,
         case (char )0x94: case (char )0x95:
         case (char )0x96: case (char )0x97:
         case (char )0x98: case (char )0x99:
-        case (char )0xa0: case (char )0xa1: case (char )0xa2: case (char )0xc0:
+        case (char )0xa0: case (char )0xa1: case (char )0xa2:
+        case (char )0xc0: case (char )0xc1:
           if (numeric) return MSERR;
           oppo++;
           if (oppo>=20) return MSERR;
@@ -1049,7 +1360,8 @@ int calculate(char *code,
           if (!numeric) return MSERR;
           mathexpand(5,&oppo,&numpo,&ifpo,&forpo,numbuf,numstat,
                      opbuf,prebuf,argbuf,noexpand,fnoexpand,
-                     memory,memorystat,sumdata,sumstat,difdata,difstat,color,marksize,
+                     memory,memorystat,sumdata,sumstat,difdata,difstat,
+                     color,marksize,marktype,
                      ufcodef,ufcodeg,ufcodeh);
           oppo++;
           if (oppo>=20) return MSERR;
@@ -1063,7 +1375,8 @@ int calculate(char *code,
           if (!numeric) return MSERR;
           mathexpand(4,&oppo,&numpo,&ifpo,&forpo,numbuf,numstat,
                      opbuf,prebuf,argbuf,noexpand,fnoexpand,
-                     memory,memorystat,sumdata,sumstat,difdata,difstat,color,marksize,
+                     memory,memorystat,sumdata,sumstat,difdata,difstat,
+                     color,marksize,marktype,
                      ufcodef,ufcodeg,ufcodeh);
           oppo++;
           if (oppo>=20) return MSERR;
@@ -1077,7 +1390,8 @@ int calculate(char *code,
           if (!numeric) return MSERR;
           mathexpand(3,&oppo,&numpo,&ifpo,&forpo,numbuf,numstat,
                      opbuf,prebuf,argbuf,noexpand,fnoexpand,
-                     memory,memorystat,sumdata,sumstat,difdata,difstat,color,marksize,
+                     memory,memorystat,sumdata,sumstat,difdata,difstat,
+                     color,marksize,marktype,
                      ufcodef,ufcodeg,ufcodeh);
           oppo++;
           if (oppo>=20) return MSERR;
@@ -1091,7 +1405,8 @@ int calculate(char *code,
           if (!numeric) return MSERR;
           mathexpand(2,&oppo,&numpo,&ifpo,&forpo,numbuf,numstat,
                      opbuf,prebuf,argbuf,noexpand,fnoexpand,
-                     memory,memorystat,sumdata,sumstat,difdata,difstat,color,marksize,
+                     memory,memorystat,sumdata,sumstat,difdata,difstat,
+                     color,marksize,marktype,
                      ufcodef,ufcodeg,ufcodeh);
           oppo++;
           if (oppo>=20) return MSERR;
@@ -1105,7 +1420,8 @@ int calculate(char *code,
           if (!numeric) return MSERR;
           mathexpand(1,&oppo,&numpo,&ifpo,&forpo,numbuf,numstat,
                      opbuf,prebuf,argbuf,noexpand,fnoexpand,
-                     memory,memorystat,sumdata,sumstat,difdata,difstat,color,marksize,
+                     memory,memorystat,sumdata,sumstat,difdata,difstat,
+                     color,marksize,marktype,
                      ufcodef,ufcodeg,ufcodeh);
           if (oppo==-1) return MSERR;
           if (argbuf[oppo]!=1) return MSERR;
@@ -1140,7 +1456,8 @@ int calculate(char *code,
           if (!numeric) return MSERR;
           mathexpand(1,&oppo,&numpo,&ifpo,&forpo,numbuf,numstat,
                      opbuf,prebuf,argbuf,noexpand,fnoexpand,
-                     memory,memorystat,sumdata,sumstat,difdata,difstat,color,marksize,
+                     memory,memorystat,sumdata,sumstat,difdata,difstat,
+                     color,marksize,marktype,
                      ufcodef,ufcodeg,ufcodeh);
           if (oppo==-1) return MSERR;
           if (argbuf[oppo]<1) return MSERR;
@@ -1200,7 +1517,8 @@ int calculate(char *code,
           if (!numeric) return MSERR;
           mathexpand(1,&oppo,&numpo,&ifpo,&forpo,numbuf,numstat,
                      opbuf,prebuf,argbuf,noexpand,fnoexpand,
-                     memory,memorystat,sumdata,sumstat,difdata,difstat,color,marksize,
+                     memory,memorystat,sumdata,sumstat,difdata,difstat,
+                     color,marksize,marktype,
                      ufcodef,ufcodeg,ufcodeh);
           if (oppo>-1) return MSERR;
           if (numpo!=0) return MSERR;
