@@ -1,21 +1,38 @@
-/**
- *
- * $Id: object.c,v 1.3 1999/04/15 12:15:27 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: object.c,v 1.6 2003/02/16 12:42:14 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: object.c,v $
+ * Revision 1.6  2003/02/16 12:42:14  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.5  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.4  2001/03/23 12:15:31  isizaka
+ * for 6.3.13
+ *
  * Revision 1.3  1999/04/15 12:15:27  isizaka
  * for release 6.03.01
  *
@@ -104,7 +121,7 @@ char *errorlist[ERRNUM]={
      "extra object arguments",
      "not enouph object argument",
      "illegal type of object argument",
-     "instance exist. can not overwrite object",
+     "instance exist. cannot overwrite object",
 };
 
 void error(struct objlist *obj,int code)
@@ -1571,7 +1588,7 @@ int _newobj(struct objlist *obj)
   objcur=obj;
   while (objcur!=NULL) {
     for (i=0;i<objcur->tblnum;i++) {
-      offset=obj->table[i].offset;
+      offset=objcur->table[i].offset;
       switch (objcur->table[i].type) {
       case NVOID:
       case NLABEL:
@@ -3218,12 +3235,12 @@ int getargument(int type,char *arglist,char *val,int *argc,
       for (i=0;i<MEMORYNUM;i++) {memory[i]=0;memorystat[i]=MNOERR;}
       rcode=calculate(code,1,
                       0,MNOERR,0,MNOERR,0,MNOERR,
-                      0,0,0,0,0,0,0,0,0,0,0,0,
+                      0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                       NULL,NULL,
                       memory,memorystat,
                       NULL,NULL,
                       NULL,NULL,
-                      NULL,NULL,
+                      NULL,NULL,NULL,
                       NULL,NULL,NULL,0,NULL,NULL,NULL,0,&vd);
       memfree(code);
       if (rcode!=MNOERR) {
@@ -3232,14 +3249,14 @@ int getargument(int type,char *arglist,char *val,int *argc,
       }
       if (arglist[alp]=='i') {
         if (arglist[1]=='a') {
-          vi=round(vd);
+          vi=nround(vd);
           if (array==NULL) {
             if ((array=arraynew(sizeof(int)))==NULL) goto errexit;
           }
           if (arrayadd(array,&vi)==NULL) goto errexit;
         } else {
           if ((p=memalloc(sizeof(int)))==NULL) goto errexit;
-          *(int *)(p)=round(vd);
+          *(int *)(p)=nround(vd);
           if (arg_add(&argv,p)==NULL) goto errexit;
         }
       } else {
