@@ -1,21 +1,38 @@
-/**
- *
- * $Id: ofit.c,v 1.4 1999/04/15 12:15:27 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: ofit.c,v 1.7 2003/02/16 12:42:14 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: ofit.c,v $
+ * Revision 1.7  2003/02/16 12:42:14  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.6  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.5  2001/03/23 12:15:31  isizaka
+ * for 6.3.13
+ *
  * Revision 1.4  1999/04/15 12:15:27  isizaka
  * for release 6.03.01
  *
@@ -34,6 +51,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <math.h>
 #include "ngraph.h"
 #include "object.h"
@@ -377,7 +395,7 @@ int fituser(struct objlist *obj,struct fitlocal *fitlocal,char *func,
     err=FALSE;
     rcode=calculate(fitlocal->codef,1,spx,MNOERR,0,0,0,0,
                     0,0,0,0,0,0,0,0,0,0,0,0,
-                    par,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
+                    par,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
                     NULL,NULL,NULL,0,NULL,NULL,NULL,0,&y1);
     if (rcode==MSERR) return -4;
     else if (rcode!=MNOERR) err=TRUE;
@@ -420,16 +438,16 @@ int fituser(struct objlist *obj,struct fitlocal *fitlocal,char *func,
       spy=data[k*2+1];
       err=FALSE;
       rcode=calculate(fitlocal->codef,1,spx,MNOERR,0,0,0,0,
-                      0,0,0,0,0,0,0,0,0,0,0,0,
-                      par,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
+                      0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+                      par,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
                       NULL,NULL,NULL,0,NULL,NULL,NULL,0,&y1);
       if (rcode==MSERR) return -4;
       else if (rcode!=MNOERR) err=TRUE;
       if (deriv) {
         for (j=0;j<dim;j++) {
           rcode=calculate(fitlocal->codedf[tbl[j]],1,spx,MNOERR,0,0,0,0,
-                          0,0,0,0,0,0,0,0,0,0,0,0,
-                          par,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
+                          0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+                      par,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
                           NULL,NULL,NULL,0,NULL,NULL,NULL,0,&(x2[j]));
           if (rcode==MSERR) return -4;
           else if (rcode!=MNOERR) err=TRUE;
@@ -441,8 +459,8 @@ int fituser(struct objlist *obj,struct fitlocal *fitlocal,char *func,
           if (dxx==0) dxx=1e-6;
           par2[tbl[j]]+=dxx;
           rcode=calculate(fitlocal->codef,1,spx,MNOERR,0,0,0,0,
-                          0,0,0,0,0,0,0,0,0,0,0,0,
-                          par2,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
+                          0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+                     par2,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
                           NULL,NULL,NULL,0,NULL,NULL,NULL,0,&(x2[j]));
           if (rcode==MSERR) return -4;
           else if (rcode!=MNOERR) err=TRUE;
@@ -532,7 +550,7 @@ int fituser(struct objlist *obj,struct fitlocal *fitlocal,char *func,
         err=FALSE;
         rcode=calculate(fitlocal->codef,1,spx,MNOERR,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,
-                        par2,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
+                     par2,parstat,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
                         NULL,NULL,NULL,0,NULL,NULL,NULL,0,&y1);
         if (rcode==MSERR) return -4;
         else if (rcode!=MNOERR) err=TRUE;
@@ -609,9 +627,17 @@ errexit:
         equation[j]=func[i];
         j++;
       } else {
-        pnum=(func[i+1]-'0')*10+(func[i+2]-'0');
+        if (isdigit(func[i+1]) && isdigit(func[i+2]) && isdigit(func[i+3])) {
+          pnum=(func[i+1]-'0')*100+(func[i+2]-'0')*10+(func[i+3]-'0');
+          i+=3;
+        } else if (isdigit(func[i+1]) && isdigit(func[i+2])) {
+          pnum=(func[i+1]-'0')*10+(func[i+2]-'0');
+          i+=2;
+        } else {
+          pnum=(func[i+1]-'0');
+          i+=1;
+        }
         j+=sprintf(equation+j,"%.15e",par[pnum]);
-        i+=2;
       }
     equation[j]='\0';
     fitlocal->equation=equation;
@@ -690,7 +716,7 @@ int fitfit(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   if (arraynum(darray)<1) return -1;
   data=arraydata(darray);
   anum=arraynum(darray)-1;
-  dnum=round(data[0]);
+  dnum=nround(data[0]);
   data+=1;
   if (dnum==(anum/2)) weight=FALSE;
   else if (dnum==(anum/3)) {
