@@ -1,21 +1,38 @@
-/**
- *
- * $Id: oprm.c,v 1.3 1999/04/15 12:15:27 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: oprm.c,v 1.6 2003/02/16 12:42:14 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: oprm.c,v $
+ * Revision 1.6  2003/02/16 12:42:14  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.5  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.4  2001/03/23 12:15:31  isizaka
+ * for 6.3.13
+ *
  * Revision 1.3  1999/04/15 12:15:27  isizaka
  * for release 6.03.01
  *
@@ -281,8 +298,12 @@ char *remarkconv(char *s,int ff,int fj,int fb,int *fnameid,char *prmfile,
         if (file==-1) file=0;
         if (toupper(s[i+3])=='N') {
           addfontcontrol(s2,&j,fchange,jchange,fff,ffb,ffj,script);
-          if (file<20)
-            j+=sprintf(s2+j,"%%{file:%d:name}",fnameid[file]);
+          if (file<20) {
+            if (s[i+3]=='N')
+              j+=sprintf(s2+j,"%%{file:%d:file}",fnameid[file]);
+            else
+              j+=sprintf(s2+j,"%%{file:%d:basename}",fnameid[file]);
+          }
           i+=3;
         } else if (s[i+3]=='d') {
           addfontcontrol(s2,&j,fchange,jchange,fff,ffb,ffj,script);
@@ -697,7 +718,7 @@ int prmload(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
       error2(obj,ERRPRM,file);
       goto errexit;
     }
-    if ((d5>=42) && (d5<=50)) {
+    if (type==19) {
       putobj(fitobj,"through_point",fitid,&d1);
       putobj(fitobj,"point_x",fitid,&f1);
       putobj(fitobj,"point_y",fitid,&f2);
@@ -1630,7 +1651,7 @@ int prmload(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
       }
       d1*=10;
       d2*=10;
-      d3=round(d3/0.21);
+      d3=nround(d3/0.21);
       putobj(mgobj,"left_margin",mgid,&d1);
       putobj(mgobj,"top_margin",mgid,&d2);
       putobj(mgobj,"zoom",mgid,&d3);
@@ -1662,7 +1683,7 @@ int prmload(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
           }
           i=0;
           do {
-            ch=fgetc(fp);
+            ch=nfgetc(fp);
             if (ch==(unsigned int)EOD[i]) i++;
             else {
               if (mkdata) for (j=0;j<i;j++) fputc(EOD[j],fp2);
@@ -1709,5 +1730,3 @@ void *addprm()
 {
   return addobject(NAME,NULL,PARENT,VERSION,TBLNUM,prm,ERRNUM,prmerrorlist,NULL,NULL);
 }
-
-
