@@ -1,21 +1,39 @@
-/**
- *
- * $Id: shell.c,v 1.3 1999/09/10 14:22:38 isizaka Exp isizaka $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: shell.c,v 1.7 2002/07/06 08:51:42 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: shell.c,v $
+ * Revision 1.7  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.5  2001/07/14 17:42:55  isizaka
+ * for 6.03.14
+ *
+ * Revision 1.4  2000/12/02 08:02:44  isizaka
+ * bug fix: pipe in cmdexec()
+ * thanks to Mr. H. Ito, hito@crl.go.jp
+ *
  * Revision 1.3  1999/09/10 14:22:38  isizaka
  * bug fix about '.' command
  *
@@ -82,7 +100,6 @@
  *        set
  *        shift [n]
  *        test
- *        touch
  *        unset
  *        [
  *        new      object [field:val, ...]
@@ -151,7 +168,7 @@ char *cmderrorlist[ERRNUM]={
   "extra arguments.",
   "not enouph argument.",
   "too many arguments.",
-  "can not open file",
+  "cannot open file",
   "Use \"exit\" to leave the shell.",
   "no such file",
   "I/O error: read file",
@@ -167,7 +184,7 @@ char *cmderrorlist[ERRNUM]={
   "non-numeric argument",
   "unknown option",
   "illegal option",
-  "can not unset",
+  "cannot unset",
   "no such directory",
   "syntax error for math.",
   "not allowd function for math.",
@@ -188,6 +205,15 @@ int security=FALSE;
 char writebuf[WRITEBUFSIZE];
 int writepo;
 
+void unlinkfile(char **file)
+{
+  if (*file!=NULL) {
+    unlink(*file);
+    free(*file);
+    *file=NULL;
+  }
+}
+
 #ifndef WINDOWS
 
 void shellevloop(int sig)
@@ -486,8 +512,6 @@ int shprintfstderr(char *fmt,...)
 
 #endif
 
-
-
 struct cmdtabletype cmdtable[CMDNUM] = {
                   {"cd",cmcd},
                   {"echo",cmecho},
@@ -495,11 +519,10 @@ struct cmdtabletype cmdtable[CMDNUM] = {
                   {"exit",cmexit} ,
                   {"export",cmexport},
                   {"pwd",cmpwd},
-                  {"read",NULL},
+                  {"read",cmread},
                   {"set",cmset},
                   {"shift",cmshift},
                   {"type",cmtype},
-                  {"touch",NULL},
                   {"unset",cmunset},
                   {":",cmtrue},
                   {"object",cmobject} ,
@@ -1106,7 +1129,7 @@ int getcmdline(struct nshell *nshell,
           eofcount=0;
           if (ch=='\n') ch='\0';
           if ((tok=nstrccat(tok,ch))==NULL) goto errexit;
-	    }
+        }
       } while (ch!='\0');
     } else {
       if (str[*istr]=='\0') {
@@ -1189,7 +1212,7 @@ char *quotation(struct nshell *nshell,char *s,int quote)
     if ((quote!='"') && (ifs!=NULL) && (ifs[0]!='\0') 
     && (strchr(ifs,s[i])!=NULL)) snew[j++]=(char )0x01;
     else {
-      if (strchr("\"\\'$",s[i])!=NULL) snew[j++]='\\';
+      if (strchr("\"\\'$",s[i])!=NULL) snew[j++]=(char )0x02;
       snew[j++]=s[i];
     }
   }
@@ -1217,7 +1240,8 @@ char *unquotation(char *s,int *quoted)
       if (!quote
       || ((quote=='"') && (strchr("\"\\'$",po[i+1])!=NULL))) escape=TRUE;
       else snew[j++]=po[i];
-    } else if (po[i]==(char )0x01) snew[j++]=' ';
+    } else if (po[i]==(char )0x02) escape=TRUE;
+    else if (po[i]==(char )0x01) snew[j++]=' ';
     else if ((quote=='"') || (quote=='\'')) { 
       if (po[i]==quote) quote='\0';
       else snew[j++]=po[i];
@@ -1253,7 +1277,8 @@ char *fnexpand(struct nshell *nshell,char *str)
       if (!quote
       || ((quote=='"') && (strchr("\"\\'$",po[i+1])!=NULL))) escape=TRUE;
       else if ((s=nstrccat(s,po[i]))==NULL) goto errexit;
-    } else if ((po[i]==(char )0x01) || (po[i]=='\0')) {
+    } else if (po[i]==(char )0x02) escape=TRUE;
+    else if ((po[i]==(char )0x01) || (po[i]=='\0')) {
       if (strlen(s)!=0) {
         if (!nshell->optionf && expand && !noexpand) {
           if ((num=nglob(s,&namelist))==-1) goto errexit;
@@ -1310,7 +1335,10 @@ int wordsplit(struct prmlist *prmcur)
   if (po==NULL) return 1;
   if ((s=nstrnew())==NULL) return -1;
   for (i=0;po[i]!='\0';i++) {
-    if (po[i]==(char )0x01) {
+    if (po[i]==(char )0x02) {
+      i++;
+      if ((s=nstrccat(s,po[i]))==NULL) goto errexit;
+    } else if (po[i]==(char )0x01) {
       prmcur->str=s;
       if ((s=nstrnew())==NULL) goto errexit;
       if ((prmnew=memalloc(sizeof(struct prmlist)))==NULL) goto errexit;
@@ -1335,11 +1363,15 @@ errexit:
 
 char *wordunsplit(char *str)
 {
-  int i;
+  int i,j;
   char *po;
 
   po=str;
-  for (i=0;po[i]!='\0';i++) if (po[i]==(char )0x01) po[i]=' ';
+  j=0;
+  for (i=0;po[i]!='\0';i++)
+    if (po[i]==(char )0x01) po[j++]=' ';
+    else if (po[i]!=(char )0x02) po[j++]=po[i];
+  po[j]='\0';
   return po;
 }
 
@@ -1394,6 +1426,11 @@ char *expand(struct nshell *nshell,char *str,int *quote,int *bquote,
           if ((s=nstrccat(s,po[i]))==NULL) goto errexit;
         }
       }
+    } else if (po[i]==(char )0x02) {
+      escape=TRUE;
+      if ((se==NULL) && (sb==NULL)) {
+        if ((s=nstrccat(s,po[i]))==NULL) goto errexit;
+      }
     /* check back quote */
     } else if (*bquote) {
       if (po[i]=='`') {
@@ -1460,8 +1497,8 @@ char *expand(struct nshell *nshell,char *str,int *quote,int *bquote,
         if ((sb=nstrnew())==NULL) goto errexit;
       }
     /* check constant variable */
-    } else if ((po[i]=='$') && (po[i+1]!='\0') && (*quote!='\'')
-    && (se==NULL)) {
+    } else if ((po[i]=='$') && (po[i+1]!='\0') 
+    && (*quote!='\'') && (se==NULL)) {
       if (isdigit(po[i+1])) {
         for (j=i+1;(po[j]!='\0') && isdigit(po[j]);j++);
         memfree(name);
@@ -1571,7 +1608,8 @@ char *expand(struct nshell *nshell,char *str,int *quote,int *bquote,
         if ((name=memalloc(j-i))==NULL) goto errexit;
         strncpy(name,po+i+1,j-i-1);
         name[j-i-1]='\0';
-        val=getval(nshell,name);
+        if (name[0]=='\0') val="$";
+        else val=getval(nshell,name);
         if (val!=NULL) {
           if ((c1=quotation(nshell,val,*quote))==NULL) goto errexit;
           s=nstrcat(s,c1);
@@ -1713,6 +1751,12 @@ int checkcmd(struct nshell *nshell,struct cmdlist **cmdroot)
   struct prmlist *prmcur,*prmprev;
   int cmd,i;
   char *po;
+  char *s,*eof;
+  int quoted,ch;
+  int eoflen,match,len;
+  struct objlist *sys;
+  char *tmpfil;
+  HANDLE sout;
 
   cmdcur=*cmdroot;
   cmdprev=NULL;
@@ -1790,13 +1834,107 @@ int checkcmd(struct nshell *nshell,struct cmdlist **cmdroot)
                 prmcur->prmno=PPSO2;
               else {
                 prmcur->prmno=PPSI2;
-	      }
+                if ((eof=unquotation((prmcur->next)->str,&quoted))==NULL)
+                  return -1;
+                memfree((prmcur->next)->str);
+                (prmcur->next)->str=eof;
+                (prmcur->next)->quoted=quoted;
+
+                /* get << contents */
+
+                if ((sys=getobject("system"))==NULL) return -1;
+                if ((tmpfil=tempnam(getval(nshell,"TMPDIR"),TEMPPFX))==NULL)
+                  return -1;
+                sout=nopen(tmpfil,O_CREAT | O_TRUNC | O_RDWR,NFMODE);
+                if ((quoted) && !nisatty(nshell->fd)) {
+                  eoflen=strlen(eof);
+                  match=0;
+                  writepo=0;
+                  while (TRUE) {
+                    ch=shget(nshell);
+                    if ((ch==EOF) || (ch=='\0')) break;
+                    if (ch=='\n') {
+                      if (match==eoflen) break;
+                      writebuf[writepo]=ch;
+                      writepo++;
+                      if (writepo==WRITEBUFSIZE) {
+                        nwrite(sout,writebuf,WRITEBUFSIZE);
+                        writepo=0;
+                      }
+                      match=0;
+                    } else if (match==-1) {
+                      writebuf[writepo]=ch;
+                      writepo++;
+                      if (writepo==WRITEBUFSIZE) {
+                        nwrite(sout,writebuf,WRITEBUFSIZE);
+                        writepo=0;
+                      }
+                    } else if ((match==eoflen) || (ch!=eof[match])) {
+                      for (i=0;i<match;i++) {
+                        writebuf[writepo]=eof[i];
+                        writepo++;
+                        if (writepo==WRITEBUFSIZE) {
+                          nwrite(sout,writebuf,WRITEBUFSIZE);
+                          writepo=0;
+                        }
+                      }
+                      writebuf[writepo]=ch;
+                      writepo++;
+                      if (writepo==WRITEBUFSIZE) {
+                        nwrite(sout,writebuf,WRITEBUFSIZE);
+                        writepo=0;
+                      }
+                      match=-1;
+                    } else match++;
+                  }
+                  if (writepo!=0) nwrite(sout,writebuf,writepo);
+                } else {
+                  do {
+                    if ((s=nstrnew())==NULL) {
+                      nclose(sout);
+                      unlinkfile(&tmpfil);
+                      return -1;
+                    }
+                    if (nisatty(nshell->fd))
+                      printfconsole("%.256s",getval(nshell,"PS2"));
+                    do {
+                      ch=shget(nshell);
+                      if (ch==EOF) {
+                        if (strlen(s)!=0) {
+                          if (!nisatty(nshell->fd)) break;
+                          else printfconsole("%c",(char )0x07);
+                        } else break;
+                      } else {
+                        if (ch=='\n') ch='\0';
+                        if ((s=nstrccat(s,ch))==NULL) {
+                          nclose(sout);
+                          unlinkfile(&tmpfil);
+                          return -1;
+                        }
+                      }
+                    } while (ch!='\0');
+                    if (strcmp0(eof,s)!=0) {
+                      len=strlen(s);
+                      s[len]='\n';
+                      nwrite(sout,s,len+1);
+                      memfree(s);
+                    } else {
+                      memfree(s);
+                      break;
+                    }
+                  } while (ch!=EOF);
+                }
+                nclose(sout);
+                memfree((prmcur->next)->str);
+                (prmcur->next)->str=tmpfil;
+              }
             } else {
               if (prmcur->str[0]=='>')
                 prmcur->prmno=PPSO1;
               else 
                 prmcur->prmno=PPSI1;
             }
+
             (prmcur->next)->prmno=prmcur->prmno;
             prmcur=prmcur->next;
           } else if (prmcur->str[0]=='|') {
@@ -1839,11 +1977,12 @@ int checkcmd(struct nshell *nshell,struct cmdlist **cmdroot)
             po=prmcur->str;
             for (i=0;(po[i]!='\0') && (isalnum(po[i]) || (po[i]=='_'));i++);
             if ((i!=0) && (po[i]=='=')) prmcur->prmno=PPSETV;
-	  }
+          }
           if (prmcur->prmno==PPNO) break;
         }
         prmcur=prmcur->next;
       }
+
       cmdprev=cmdcur;
       cmdcur=cmdcur->next;
     }
@@ -2107,31 +2246,28 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
    struct cmdstack *stroot,*stcur,*st;
    char *fstdout,*fstdin;
    int istdout,istdin;
-   HANDLE sout,sout2,sin,sin2,sin3;
-   int quoted,lastc;
+   HANDLE sout,sout2,sin,sin2,sin3,fd;
+   int lastc;
    int pipef;
    struct objlist *sys;
    char *tmpfil,*tmpfil2;
    char *cmds;
-   int ch;
    char *s;
    char **argv,**newenviron;
    char **argvsave,**argvnew;
+   int quoted;
    int argcsave,argcnew;
    int iftrue,casetrue;
    char *pat;
-   int match,eoflen;
    char *readbuf;
    int readpo;
    int readbyte;
+   int ch;
+   char buf[2];
+  
 #ifndef WINDOWS
-   int pid;
+   pid_t pid;
 #else
-//   char *CommandLine;
-//   STARTUPINFO si;
-//   PROCESS_INFORMATION pi;
-//   DWORD ExitCode;
-//   HANDLE dStdInput,dStdOutput,dStdError;
    int pid;
    WaitPidParam WP;
    DWORD IDThread;
@@ -2427,8 +2563,8 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
            }
            prmcur->quoted=FALSE;
          } else {
-           if ((str=unquotation(prmcur->str,&quoted))==NULL) goto errexit;
-           prmcur->quoted=quoted;
+           if ((str=memalloc(strlen(prmcur->str)+1))==NULL) goto errexit;
+           strcpy(str,prmcur->str);
          }
          if ((prmcur->prmno!=PPNO) || (str[0]!='\0')) {
            if ((prm=memalloc(sizeof(struct prmlist)))==NULL) goto errexit;
@@ -2458,7 +2594,6 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
          prmcur=prmcur->next;
        }
 
-
        /* remove null parameter & check redirect and pipe */
        istdout=istdin=PPNO;
        pipef=FALSE;
@@ -2605,90 +2740,54 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
          if (istdin!=PPNO) {
            if (istdin==PPSI1) sin=nopen(fstdin,O_RDONLY,NFMODE);
            else if (istdin==PPSI2) {
-             if ((sys=getobject("system"))==NULL) goto errexit;
-             if ((tmpfil2=tempnam(getval(nshell,"TMPDIR"),TEMPPFX))==NULL)
-               goto errexit;
-             sin=nopen(tmpfil2,O_CREAT | O_TRUNC | O_RDWR,NFMODE);
-             if ((quoted) && (!nisatty(nshell->fd))) {
-               eoflen=strlen(fstdin);
-               match=0;
-               writepo=0;
-               while (TRUE) {
-                 ch=shget(nshell);
-                 if ((ch==EOF) || (ch=='\0')) break;
-                 if (ch=='\n') {
-                   if (match==eoflen) break;
-                   writebuf[writepo]=ch;
-                   writepo++;
-                   if (writepo==WRITEBUFSIZE) {
-                     nwrite(sin,writebuf,WRITEBUFSIZE);
-                     writepo=0;
-                   }
-                   match=0;
-                 } else if (match==-1) {
-                   writebuf[writepo]=ch;
-                   writepo++;
-                   if (writepo==WRITEBUFSIZE) {
-                     nwrite(sin,writebuf,WRITEBUFSIZE);
-                     writepo=0;
-                   }
+             sin=nopen(fstdin,O_RDONLY,NFMODE);
+             if (!quoted) {
+               if ((tmpfil2=tempnam(getval(nshell,"TMPDIR"),TEMPPFX))==NULL) {
+                 nclose(sin);
+                 goto errexit;
+               }
+               fd=nopen(tmpfil2,O_CREAT | O_TRUNC | O_RDWR,NFMODE);
+               do {
+                 if ((s=nstrnew())==NULL) {
+                   nclose(sin);
+                   nclose(fd);
+                   goto errexit;
                  }
-                 else if ((match==eoflen) || (ch!=fstdin[match])) {
-                   for (i=0;i<match;i++) {
-                     writebuf[writepo]=fstdin[i];
-                     writepo++;
-                     if (writepo==WRITEBUFSIZE) {
-                       nwrite(sin,writebuf,WRITEBUFSIZE);
-                       writepo=0;
+                 do {
+                   if (nread(sin,buf,1)>0) ch=buf[0];
+                   else ch=EOF;
+                   if (ch==EOF) break;
+                   else {
+                     if (ch=='\n') ch='\0';
+                     if ((s=nstrccat(s,ch))==NULL) {
+                       nclose(sin);
+                       nclose(fd);
+                       goto errexit;
                      }
                    }
-                   writebuf[writepo]=ch;
-                   writepo++;
-                   if (writepo==WRITEBUFSIZE) {
-                     nwrite(sin,writebuf,WRITEBUFSIZE);
-                     writepo=0;
-                   }
-                   match=-1;
-                 } else match++;
-               }
-               if (writepo!=0) nwrite(sin,writebuf,writepo);
-             } else {
-               do {
-                 if ((s=nstrnew())==NULL) goto errexit;
-                 if (nisatty(nshell->fd))
-                   printfconsole("%.256s",getval(nshell,"PS2"));
-                 do {
-                   ch=shget(nshell);
-                   if (ch==EOF) {
-                     if (strlen(s)!=0) {
-                       if (!nisatty(nshell->fd)) break;
-                       else printfconsole("%c",(char )0x07);
-                     } else break;
-                   } else {
-                     if (ch=='\n') ch='\0';
-                     if ((s=nstrccat(s,ch))==NULL) goto errexit;
-                   }
                  } while (ch!='\0');
-                 if (strcmp0(fstdin,s)!=0) {
-                   if (!quoted) {
-                     quote=bquote='\0';
-                     str=s;
-                     s=expand(nshell,str,&quote,&bquote,FALSE);
-                     memfree(str);
-                     if (s==NULL) goto errexit;
-                     wordunsplit(s);
-                   }
-                   len=strlen(s);
-                   s[len]='\n';
-                   nwrite(sin,s,len+1);
-                   memfree(s);
-                 } else {
-                   memfree(s);
-                   break;
+                 quote=bquote='\0';
+                 str=s;
+                 s=expand(nshell,str,&quote,&bquote,FALSE);
+                 memfree(str);
+                 if (s==NULL) {
+                   nclose(sin);
+                   nclose(fd);
+                   goto errexit;
                  }
+                 wordunsplit(s);
+                 len=strlen(s);
+                 if (ch!=EOF) {
+                   s[len]='\n';
+                   len++;
+                 }
+                 nwrite(fd,s,len);
+                 memfree(s);
                } while (ch!=EOF);
+               nclose(sin);
+               nclose(fd);
+               sin=nopen(tmpfil2,O_RDONLY,NFMODE);
              }
-             nlseek(sin,0L,SEEK_SET);
            }
          } else if (cmdcur->pipefile!=NULL) {
            sin=nopen(cmdcur->pipefile,O_RDONLY,NFMODE);
@@ -2710,6 +2809,7 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
            if ((tmpfil=tempnam(getval(nshell,"TMPDIR"),TEMPPFX))==NULL)
              goto errexit;
            sout=nopen(tmpfil,O_CREAT | O_TRUNC | O_RDWR,NFMODE);
+           unlinkfile(&((cmdcur->next)->pipefile));
            (cmdcur->next)->pipefile=tmpfil;
          }
          if (sout!=NOHANDLE) sout2=nredirect(1,sout);
@@ -2763,7 +2863,7 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
            nshell->argc=argcnew;
            nshell->argv=argvnew;
            rcode=cmdexec(nshell,cmdnew,TRUE);
-           arg_del(argvnew);
+           arg_del(nshell->argv);
            nshell->argc=argcsave;
            nshell->argv=argvsave;
            if ((rcode==-1) || (rcode==1)) goto errexit;
@@ -2861,7 +2961,7 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
                  a--;
 	       } else cmdstackrmlast(&stroot);
 	     }
-	   }
+           }
          /* continue */
          } else if (strcmp0("continue",cmds)==0) {
            if (pnum>2) {
@@ -2895,10 +2995,10 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
                      stcur->prm=NULL;
                    }
                    cmdstackrmlast(&stroot);
-		 }
-	       } else cmdstackrmlast(&stroot);
-	     }
-	   }
+                 }
+               } else cmdstackrmlast(&stroot);
+             }
+           }
          /* return */
          } else if (strcmp0("return",cmds)==0) {
            if (pnum>2) {
@@ -2968,7 +3068,7 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
                  if (arg_add(&argv,prm->str)==NULL) {
                    memfree(argv);
                    goto errexit;
-		         }
+                 }
                  prm=prm->next;
                }
                errlevel=cmdtable[i].proc(nshell,pnum,(char **)argv);
@@ -2991,7 +3091,7 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
                  if (arg_add(&argv,prm->str)==NULL) {
                    memfree(argv);
                    goto errexit;
-	             }
+                 }
                  prm=prm->next;
                }
 #ifndef WINDOWS
@@ -3005,7 +3105,7 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
                               argv[0],strerror(errno));
                  exit(errlevel);
                } else {
-                 while (waitpid(pid,&errlevel,WNOHANG)!=pid) eventloop();
+                 while (waitpid(pid,&errlevel,WNOHANG)==0) eventloop();
                }
 #else
                pid=spawnve(P_NOWAIT,cmdname,(char **)argv,mainenviron);
@@ -3019,12 +3119,13 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
                  while (waitpid(&WP)!=pid) eventloop();
                  errlevel=WP.errlevel;
                  errlevel=errlevel >>8;
+                 nsetconsolemode();
                }
 #endif
                memfree(argv);
                memfree(cmdname);
                cmdname=NULL;
-	         }
+             }
            }
            nshell->status=errlevel;
            lastc=cmdstackgetlast(&stroot);
@@ -3045,11 +3146,7 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
            nredirect2(0,sin2);
            sin2=NOHANDLE;
          }
-         if (tmpfil2!=NULL) {
-           unlink(tmpfil2);
-           free(tmpfil2);
-           tmpfil2=NULL;
-         }
+         unlinkfile(&tmpfil2);
          arg_del(mainenviron);
          mainenviron=environ;
          environ=NULL;
@@ -3062,9 +3159,12 @@ int cmdexec(struct nshell *nshell,struct cmdlist *cmdroot,int namedfunc)
        memfree(fstdout);
        memfree(fstdin);
        fstdout=fstdin=NULL;
+       unlinkfile(&(cmdcur->pipefile));
        cmdcur=cmdcur->next;
        break;
      default:
+
+       unlinkfile(&(cmdcur->pipefile));
        cmdcur=cmdcur->next;
        break;
      }
@@ -3084,19 +3184,20 @@ errexit:
   }
   if (newvalroot!=NULL) restoreval(nshell,newvalroot);
   if (sout2!=NOHANDLE) nredirect2(1,sout2);
-  if (sin2!=NOHANDLE) {
-    nredirect2(0,sin2);
-    if (tmpfil2!=NULL) {
-      unlink(tmpfil2);
-      free(tmpfil2);
-    }
-  }
+  if (sin2!=NOHANDLE) nredirect2(0,sin2);
+  unlinkfile(&tmpfil2);
   cmdcur=cmdroot;
   while (cmdcur!=NULL) {
-    if (cmdcur->pipefile!=NULL) {
-      unlink(cmdcur->pipefile);
-      free(cmdcur->pipefile);
+    prmcur=cmdcur->prm;
+    while (prmcur!=NULL) {
+      if ((prmcur->prmno==PPSI2) && (prmcur->next!=NULL)
+      && ((prmcur->next)->str!=NULL)) {
+        unlink((prmcur->next)->str);
+        prmcur=prmcur->next;
+      }
+      prmcur=prmcur->next;
     }
+    unlinkfile(&(cmdcur->pipefile));
     cmdcur=cmdcur->next;
   }
   nshell->cmdexec--;
