@@ -1,21 +1,38 @@
-/**
- *
- * $Id: shellcm.c,v 1.3 1999/04/15 12:15:27 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: shellcm.c,v 1.6 2003/02/16 12:42:14 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: shellcm.c,v $
+ * Revision 1.6  2003/02/16 12:42:14  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.5  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.4  2001/03/23 12:15:31  isizaka
+ * for 6.3.13
+ *
  * Revision 1.3  1999/04/15 12:15:27  isizaka
  * for release 6.03.01
  *
@@ -409,13 +426,21 @@ void dispfield(struct objlist *obj,char *name)
     if ((alist=chkobjarglist(obj,name))!=NULL) {
       if (alist[0]=='\0') printfstdout(" void");
       else
-        for (j=0;alist[j]!='\0';j++)
+        for (j=0;alist[j]!='\0';j++) {
           switch (alist[j]) {
+          case 'b': printfstdout(" bool"); break;
+          case 'c': printfstdout(" char"); break;
           case 'i': printfstdout(" int"); break;
           case 'd': printfstdout(" double"); break;
           case 's': printfstdout(" char *"); break;
+          case 'p': printfstdout(" void *"); break;
           case 'o': printfstdout(" obj"); break;
           }
+          if (alist[j+1]=='a') {
+            printfstdout("[]");
+            j++;
+          }
+        }
     }
     printfstdout(" )");
   } else if (ftype==NENUM) {
@@ -947,12 +972,12 @@ int cmdexpr(struct nshell*nshell,int argc,char **argv)
   for (i=0;i<MEMORYNUM;i++) {memory[i]=0;memorystat[i]=MNOERR;}
   rcode=calculate(code,1,
                   0,MNOERR,0,MNOERR,0,MNOERR,
-                  0,0,0,0,0,0,0,0,0,0,0,0,
+                  0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                   NULL,NULL,
                   memory,memorystat,
                   NULL,NULL,
                   NULL,NULL,
-                  NULL,NULL,
+                  NULL,NULL,NULL,
                   NULL,NULL,NULL,0,NULL,NULL,NULL,0,&vd);
   memfree(code);
   ecode=0;
@@ -970,16 +995,36 @@ int cmdexpr(struct nshell*nshell,int argc,char **argv)
     return ERR;
   }
   if (argv[0][0]=='d') printfstdout("%.15e\n",vd);
-  else printfstdout("%d\n",round(vd));
+  else printfstdout("%d\n",nround(vd));
   return 0;
 }
 
+int cmread(struct nshell *nshell,int argc,char **argv)
+{
+  int c,i,len;
+  char *s,*po,*s2,*ifs;
 
-
-
-
-
-
-
-
-
+  if ((s=nstrnew())==NULL) return ERR;
+  while (TRUE) {
+    c=getstdin();
+    if ((c=='\n') || (c==EOF)) break;
+    if ((s=nstrccat(s,c))==NULL) return ERR;
+  }
+  if (argc==1) {
+    addval(nshell,"REPLY",s);
+  } else {
+    po=s;
+    ifs=getval(nshell,"IFS");
+    for (i=1;i<argc;i++) {
+      if ((s2=getitok2(&po,&len,ifs))!=NULL) {
+        addval(nshell,argv[i],s2);
+        memfree(s2);
+      } else {
+        addval(nshell,argv[i],"");
+      }
+    }
+  }
+  memfree(s);
+  if (c==EOF) return ERR;
+  else return 0;
+}
