@@ -1,21 +1,41 @@
-/**
- *
- * $Id: ogra2x11.c,v 1.3 1999/04/15 12:14:26 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: ogra2x11.c,v 1.7 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: ogra2x11.c,v $
+ * Revision 1.7  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.6  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.5  2001/07/14 17:44:50  isizaka
+ * for 6.03.14
+ *
+ * Revision 1.4  2001/03/23 12:17:43  isizaka
+ * for 6.3.13
+ *
  * Revision 1.3  1999/04/15 12:14:26  isizaka
  * for release 6.03.01
  *
@@ -71,18 +91,21 @@ extern int OpenApplication();
 #define ERRNUM 2
 
 char *x11errorlist[ERRNUM]={
-  "can not open display.",
+  "cannot open display.",
   "not enough color cell."
 };
 
 #define X11FONTCASH 20 /* must be greater than 1 */
 #define X11COLORDEPTH 2
 
+enum {NORMAL=0,BOLD,ITALIC,BOLDITALIC};
+
 struct fontmap;
 
 struct fontmap {
   char *fontalias;
   char *fontname;
+  int type;
   int twobyte;
   struct fontmap *next;
 };
@@ -91,7 +114,8 @@ struct fontlocal {
   char *fontalias;
   Font font;
   XFontStruct *fontstruct;
-  int fontsize,fontdir;
+  int fontsize,fonttype,fontdir;
+  int iso8859;
 };
 
 struct x11local {
@@ -114,19 +138,20 @@ struct x11local {
   int loadfont;
   int loadfontf;
   char *fontalias;
-  double fontspace,fontcos,fontsin;
+  double fontsize,fontspace,fontcos,fontsin;
   struct fontlocal font[X11FONTCASH];
   XPoint points[LINETOLIMIT];
   int linetonum;
   int PaperWidth,PaperHeight;
   int backingstore;
+  int minus_hyphen;
 };
 
 int x11loadconfig(struct objlist *obj,struct x11local *x11local)
 {
   FILE *fp;
   char *tok,*str,*s2;
-  char *f1,*f2,*f3;
+  char *f1,*f2,*f3,*f4;
   struct fontmap *fcur,*fnew;
   int val;
   char *endptr;
@@ -140,12 +165,15 @@ int x11loadconfig(struct objlist *obj,struct x11local *x11local)
       f1=getitok2(&s2,&len," \t,");
       f2=getitok2(&s2,&len," \t,");
       f3=getitok2(&s2,&len," \t,");
-      if ((f1!=NULL) && (f2!=NULL) && (f3!=NULL)) {
+      for (;(s2[0]!='\0') && (strchr(" \x09,",s2[0])!=NULL);s2++);
+      f4=getitok2(&s2,&len,"");
+      if ((f1!=NULL) && (f2!=NULL) && (f3!=NULL) && (f4!=NULL)) {
         if ((fnew=memalloc(sizeof(struct fontmap)))==NULL) {
           memfree(tok);
           memfree(f1);
           memfree(f2);
           memfree(f3);
+          memfree(f4);
           closeconfig(fp);
           return 1;
         }
@@ -153,14 +181,20 @@ int x11loadconfig(struct objlist *obj,struct x11local *x11local)
         else fcur->next=fnew;
         fcur=fnew;
         fcur->next=NULL;
-        val=strtol(f2,&endptr,10);
         fcur->fontalias=f1;
+        if (strcmp(f2,"bold")==0) fcur->type=BOLD;
+        else if (strcmp(f2,"italic")==0) fcur->type=ITALIC;
+        else if (strcmp(f2,"bold_italic")==0) fcur->type=BOLDITALIC;
+        else fcur->type=NORMAL;
+        memfree(f2);
+        val=strtol(f3,&endptr,10);
         fcur->twobyte=val;
-        fcur->fontname=f3;
+        fcur->fontname=f4;
       } else {
         memfree(f1);
         memfree(f2);
         memfree(f3);
+        memfree(f4);
       }
     } else if (strcmp(tok,"backing_store")==0) {
       f1=getitok2(&s2,&len," \t,");
@@ -195,6 +229,11 @@ int x11loadconfig(struct objlist *obj,struct x11local *x11local)
         else x11local->autoredraw=TRUE;
       }
       memfree(f1);
+    } else if (strcmp(tok,"minus_hyphen")==0) {
+      f1=getitok2(&s2,&len," \x09,");
+      val=strtol(f1,&endptr,10);
+      if (endptr[0]=='\0') x11local->minus_hyphen=val;
+      memfree(f1);
     }
     memfree(tok);
     memfree(str);
@@ -208,7 +247,7 @@ int x11_evloop(struct objlist *obj,char *inst,char *rval,int argc,char **argv);
 void x11_redraw(struct objlist *obj,void *inst,struct x11local *x11local)
 {
   GRAredraw(obj,inst,FALSE,FALSE);
-  XFlush(Disp); 
+  x11flush(obj,inst,NULL,0,NULL);
 }
 
 void x11evpaint(Widget w,XtPointer client_data,XtPointer call_data)
@@ -224,8 +263,8 @@ void x11evpaint(Widget w,XtPointer client_data,XtPointer call_data)
   if (!(x11local->autoredraw)) return;
   x11local->scrollx=x11local->hscroll;
   x11local->scrolly=x11local->vscroll;
-  x11_redraw(x11local->obj,x11local->inst,x11local);
   x11MakeRuler(x11local);
+  x11_redraw(x11local->obj,x11local->inst,x11local);
 }
 
 void x11close(Widget w,XtPointer client_data,XtPointer call_data)
@@ -348,6 +387,7 @@ int x11init(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   x11local->autoredraw=TRUE;
   x11local->cdepth=X11COLORDEPTH;
   x11local->backingstore=FALSE;
+  x11local->minus_hyphen=TRUE;
   if (x11loadconfig(obj,x11local)) goto errexit;
   if (x11local->windpi<1) x11local->windpi=1;
   if (x11local->windpi>2540) x11local->windpi=2540;
@@ -374,11 +414,10 @@ int x11init(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   XtSetArg(al[ac],XmNcommandWindowLocation,XmCOMMAND_ABOVE_WORKSPACE); ac++; 
   XtSetArg(al[ac],XmNheight,x11local->winheight); ac++;
   XtSetArg(al[ac],XmNwidth,x11local->winwidth); ac++;
-  x11local->mainwin=XmCreateMainWindow(toplevel,"mainwin",al,ac);
+  x11local->mainwin=XmCreateMainWindow(toplevel,"gra2x11",al,ac);
   XtManageChild(x11local->mainwin);
   ac=0;
-  XtSetArg(al[ac],XmNbackground,WhitePixel(Disp,0)); ac++;
-  x11local->View=XmCreateDrawingArea(x11local->mainwin,NULL,al,ac);
+  x11local->View=XmCreateDrawingArea(x11local->mainwin,"viewer",al,ac);
   XtManageChild(x11local->View); 
   ac=0;
   XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
@@ -390,7 +429,17 @@ int x11init(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   XtManageChild(x11local->VScroll);
   XmMainWindowSetAreas(x11local->mainwin,NULL,NULL,
                        x11local->HScroll,x11local->VScroll,x11local->View); 
+
+#ifdef PRIVATECOLOR
+  x11local->cmap=XCreateColormap(Disp,RootWindow(Disp,0),
+                                 DefaultVisual(Disp,0),AllocNone);
+#else
+  x11local->cmap=DefaultColormap(Disp,0);
+#endif
+
   XtRealizeWidget(toplevel);
+  XSetWindowColormap(Disp,XtWindow(toplevel),x11local->cmap);
+
   win=XtWindow(x11local->View);
   x11local->title=mkobjlist(obj,NULL,oid,NULL,TRUE);
   XtVaSetValues(toplevel,XmNtitle,x11local->title,NULL);
@@ -416,8 +465,6 @@ int x11init(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   x11local->loadfontf=-1;
   x11local->linetonum=0;
 
-  x11local->cmap=DefaultColormap(Disp,0);
-
   XSync(Disp,FALSE);
 
   if (chkobjfield(obj,"_evloop")) goto errexit;
@@ -598,32 +645,32 @@ int x11_evloop(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
 
 int dot2pixel(struct x11local *x11local,int r)
 {
-  return round(r*x11local->pixel_dot);
+  return nround(r*x11local->pixel_dot);
 }
 
 int dot2pixelx(struct x11local *x11local,int x)
 {
-  return round(x*x11local->pixel_dot+x11local->offsetx-x11local->scrollx);
+  return nround(x*x11local->pixel_dot+x11local->offsetx-x11local->scrollx);
 }
 
 int dot2pixely(struct x11local *x11local,int y)
 {
-  return round(y*x11local->pixel_dot+x11local->offsety-x11local->scrolly);
+  return nround(y*x11local->pixel_dot+x11local->offsety-x11local->scrolly);
 }
 
 int pixel2dot(struct x11local *x11local,int r)
 {
-  return round(r/x11local->pixel_dot);
+  return nround(r/x11local->pixel_dot);
 }
 
 int pixel2dotx(struct x11local *x11local,int x)
 {
-  return round((x-x11local->offsetx)/x11local->pixel_dot);
+  return nround((x-x11local->offsetx)/x11local->pixel_dot);
 }
 
 int pixel2doty(struct x11local *x11local,int y)
 {
-  return round((y-x11local->offsety)/x11local->pixel_dot);
+  return nround((y-x11local->offsety)/x11local->pixel_dot);
 }
 
 void x11changedpi(struct x11local *x11local)
@@ -697,14 +744,16 @@ void x11MakeRuler(struct x11local *x11local)
   disp=Disp;
   win=x11local->win;
   gc=XCreateGC(Disp,x11local->win,0,0);
+  XSetFunction(Disp,gc,GXxor);
   XSetLineAttributes(disp,gc,1,LineSolid,CapButt,JoinMiter);
-  col1=x11RGB(x11local,127,127,127);
+  col1=WhitePixel(Disp,0) ^ x11RGB(x11local,127,127,127);
   XSetForeground(disp,gc,col1);
   XDrawRectangle(disp,win,gc,
                  dot2pixelx(x11local,0)-x11local->offsetx,
                  dot2pixely(x11local,0)-x11local->offsety,
                  dot2pixelx(x11local,Width)-dot2pixelx(x11local,0),
                  dot2pixely(x11local,Height)-dot2pixely(x11local,0));
+  XSetFunction(Disp,gc,GXcopy);
   XFreeGC(Disp,gc);
 }
 
@@ -719,12 +768,12 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
   char **fname2,**fname3;
   struct fontlocal font;
   struct fontmap *fcur;
-  char fontmatrix[128];
+  char fontmatrix[256];
   int rcount2,rcount3;
   Font newfont;
   XFontStruct *newfontstruct;
   int store;
-  int twobyte=FALSE;
+  int twobyte=FALSE,type;
 
   fontcashfind=-1;
   for (i=0;i<x11local->loadfont;i++) {
@@ -750,6 +799,7 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
   while (fcur!=NULL) {
     if (strcmp(fontalias,fcur->fontalias)==0) {
       fontname=fcur->fontname;
+      type=fcur->type;
       twobyte=fcur->twobyte;
       break;
     }
@@ -770,8 +820,14 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
   strncpy(fontname3,fontname,fsp-fontname+1);
   fontname2[fsp-fontname+1]='\0';
   fontname3[fsp-fontname+1]='\0';
-  sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
-          size*fcos,size*fsin,-size*fsin,size*fcos);
+  if ((type==ITALIC) || (type==BOLDITALIC)) {
+    sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
+            size*fcos,size*fsin,
+           -size*fsin+size*fcos*0.36,size*fcos+size*fsin*0.36);
+  } else {
+    sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
+            size*fcos,size*fsin,-size*fsin,size*fcos);
+  }
   for (i=0;fontmatrix[i]!='\0';i++)
     if (fontmatrix[i]=='-') fontmatrix[i]='~';
   strcat(fontname2,fontmatrix);
@@ -813,7 +869,12 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
     (x11local->font[store]).font=newfont;
     (x11local->font[store]).fontstruct=newfontstruct;
     (x11local->font[store]).fontsize=size;
+    (x11local->font[store]).fonttype=type;
     (x11local->font[store]).fontdir=dir;
+    if (strcmp(fname2[0]+strlen(fname2[0])-9,"iso8859-1")==0)
+      (x11local->font[store]).iso8859=TRUE;
+    else
+      (x11local->font[store]).iso8859=FALSE;
     if (top) XSetFont(disp,gc,(x11local->font[store]).font);
     x11local->loadfont++;
     if (fname2!=NULL) XFreeFontNames(fname2);
@@ -832,6 +893,7 @@ int x11_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   int *cpar;
   char *cstr,ch[1];
   int i,x,y,x1,y1,x2,y2;
+  double bs;
   XRectangle rect;
   int width,style,cap,join,dashn=0;
   char *dashlist=NULL;
@@ -1033,6 +1095,7 @@ int x11_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
     fontspace=cpar[2]/72.0*25.4;
     x11local->fontspace=fontspace;
     fontsize=cpar[1]/72.0*25.4;
+    x11local->fontsize=fontsize;
     fontcashsize=dot2pixel(x11local,fontsize);
     fontcashdir=cpar[3];
     fontdir=cpar[3]*MPI/18000.0;
@@ -1066,19 +1129,36 @@ int x11_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         i++;
       }
       XDrawString(Disp,x11local->win,x11local->gc,
-                  dot2pixelx(x11local,round(x0)),
-                  dot2pixely(x11local,round(y0)),ch,1);
+                  dot2pixelx(x11local,nround(x0)),
+                  dot2pixely(x11local,nround(y0)),ch,1);
+      if ((x11local->font[0].fonttype==BOLD) 
+       || (x11local->font[0].fonttype==BOLDITALIC)) {
+        bs=x11local->fontsize*0.018;
+        XDrawString(Disp,x11local->win,x11local->gc,
+          dot2pixelx(x11local,nround(x0+bs)),
+          dot2pixely(x11local,nround(y0)),ch,1); 
+        XDrawString(Disp,x11local->win,x11local->gc,
+          dot2pixelx(x11local,nround(x0)),
+          dot2pixely(x11local,nround(y0-bs)),ch,1); 
+        XDrawString(Disp,x11local->win,x11local->gc,
+          dot2pixelx(x11local,nround(x0+bs)),
+          dot2pixely(x11local,nround(y0-bs)),ch,1); 
+      }
       if ((x11local->font[0]).fontstruct==NULL) {
         fontwidth=pixel2dot(x11local,(x11local->font[0]).fontsize);
       } else {
+        if ((x11local->font[0]).iso8859 && x11local->minus_hyphen
+        && (ch[0]=='-')) {
+          ch[0]='+';
+        }
         fontwidth=pixel2dot(x11local,
                   XTextWidth((x11local->font[0]).fontstruct,ch,1));
       }
       x0+=(fontwidth+x11local->fontspace)*x11local->fontcos;
       y0-=(fontwidth+x11local->fontspace)*x11local->fontsin;
     }
-    x11local->cpx=round(x0);
-    x11local->cpy=round(y0);
+    x11local->cpx=nround(x0);
+    x11local->cpy=nround(y0);
     break;
   case 'K':
     if (x11local->loadfontf==-1) break;
@@ -1091,8 +1171,21 @@ int x11_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         kanji[0].byte1=jis >> 8;
         kanji[0].byte2=jis & 0xff;
         XDrawString16(Disp,x11local->win,x11local->gc,
-                  dot2pixelx(x11local,round(x0)),
-                  dot2pixely(x11local,round(y0)),kanji,1);
+                  dot2pixelx(x11local,nround(x0)),
+                  dot2pixely(x11local,nround(y0)),kanji,1);
+        if ((x11local->font[0].fonttype==BOLD) 
+         || (x11local->font[0].fonttype==BOLDITALIC)) {
+          bs=x11local->fontsize*0.018;
+          XDrawString16(Disp,x11local->win,x11local->gc,
+                    dot2pixelx(x11local,nround(x0+bs)),
+                    dot2pixely(x11local,nround(y0)),kanji,1);
+          XDrawString16(Disp,x11local->win,x11local->gc,
+                    dot2pixelx(x11local,nround(x0)),
+                    dot2pixely(x11local,nround(y0-bs)),kanji,1);
+          XDrawString16(Disp,x11local->win,x11local->gc,
+                    dot2pixelx(x11local,nround(x0+bs)),
+                    dot2pixely(x11local,nround(y0-bs)),kanji,1);
+        }
         if ((x11local->font[0]).fontstruct==NULL) {
           fontwidth=pixel2dot(x11local,(x11local->font[0]).fontsize);
         } else {
@@ -1104,8 +1197,8 @@ int x11_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         i+=2;
       } else i++;
     }
-    x11local->cpx=round(x0);
-    x11local->cpy=round(y0);
+    x11local->cpx=nround(x0);
+    x11local->cpy=nround(y0);
     break;
   default: break;
   }
@@ -1137,7 +1230,7 @@ int x11_charwidth(struct objlist *obj,char *inst,char *rval,
     fontstruct=(x11local->font[cashpos]).fontstruct;
     if (fontstruct==NULL) {
       if (ch[1]!=0) *(int *)rval=size;
-      else *(int *)rval=round(size*0.600);
+      else *(int *)rval=nround(size*0.600);
     } else {
       if (ch[1]!=0) {
         jis=njms2jis(*(unsigned int *)(argv[3]));
@@ -1145,11 +1238,15 @@ int x11_charwidth(struct objlist *obj,char *inst,char *rval,
         kanji[0].byte2=jis & 0xff;
         *(int *)rval=pixel2dot(x11local,XTextWidth16(fontstruct,kanji,1));
       } else {
+        if ((x11local->font[cashpos]).iso8859 && x11local->minus_hyphen
+        && (ch[0]=='-')) {
+          ch[0]='+';
+        }
         *(int *)rval=pixel2dot(x11local,XTextWidth(fontstruct,ch,1));
       }
     }
   } else {
-    *(int *)rval=round(size*0.600);
+    *(int *)rval=nround(size*0.600);
   }
   return 0;
 }
@@ -1189,19 +1286,19 @@ int x11_charheight(struct objlist *obj,char *inst,char *rval,
     fontstruct=(x11local->font[cashpos]).fontstruct;
     if (fontstruct==NULL) {
       if (twobyte) {
-        if (height) *(int *)rval=round(size*0.791);
-        else *(int *)rval=round(size*0.250);
+        if (height) *(int *)rval=nround(size*0.791);
+        else *(int *)rval=nround(size*0.250);
       } else {
-        if (height) *(int *)rval=round(size*0.562);
-        else *(int *)rval=round(size*0.250);
+        if (height) *(int *)rval=nround(size*0.562);
+        else *(int *)rval=nround(size*0.250);
       }
     } else {
       if (height) *(int *)rval=pixel2dot(x11local,fontstruct->ascent);
       else *(int *)rval=pixel2dot(x11local,fontstruct->descent);
     }
   } else {
-    if (height) *(int *)rval=round(size*0.562);
-    else *(int *)rval=round(size*0.250);
+    if (height) *(int *)rval=nround(size*0.562);
+    else *(int *)rval=nround(size*0.250);
   }
   return 0;
 }
