@@ -1,21 +1,41 @@
-/**
- *
- * $Id: ox11menu.c,v 1.3 1999/04/15 12:14:26 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: ox11menu.c,v 1.7 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: ox11menu.c,v $
+ * Revision 1.7  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.6  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.5  2001/07/14 17:44:50  isizaka
+ * for 6.03.14
+ *
+ * Revision 1.4  2001/03/23 12:17:43  isizaka
+ * for 6.3.13
+ *
  * Revision 1.3  1999/04/15 12:14:26  isizaka
  * for release 6.03.01
  *
@@ -66,7 +86,7 @@
 char *menuerrorlist[ERRNUM]={
   "already running.",
   "not enough color cell.",
-  "can not create font. Check `windowfont' resource.",
+  "cannot create font. Check `windowfont' resource.",
 };
 
 extern int OpenApplication();
@@ -109,6 +129,13 @@ int mx11loadconfig()
         if (endptr[0]=='\0') menulocal.scriptconsole=val;
       }
       memfree(f1);
+    } else if (strcmp(tok,"addin_console")==0) {
+      f1=getitok2(&s2,&len," \t,");
+      if (f1!=NULL) {
+        val=strtol(f1,&endptr,10);
+        if (endptr[0]=='\0') menulocal.addinconsole=val;
+      }
+      memfree(f1);
     } else if (strcmp(tok,"change_directory")==0) {
       f1=getitok2(&s2,&len," \t,");
       if (f1!=NULL) {
@@ -123,6 +150,26 @@ int mx11loadconfig()
         if (endptr[0]=='\0') menulocal.savehistory=val;
       }
       memfree(f1);
+    } else if (strcmp(tok,"expand_dir")==0) {
+      f1=getitok2(&s2,&len,"");
+      if (f1!=NULL) {
+        memfree(menulocal.expanddir);
+        menulocal.expanddir=f1;
+      }
+    } else if (strcmp(tok,"expand")==0) {
+      f1=getitok2(&s2,&len," \x09,");
+      if (f1!=NULL) {
+        val=strtol(f1,&endptr,10);
+        if (endptr[0]=='\0') menulocal.expand=val;
+      }
+      memfree(f1);
+    } else if (strcmp(tok,"ignore_path")==0) {
+      f1=getitok2(&s2,&len," \x09,");
+      if (f1!=NULL) {
+        val=strtol(f1,&endptr,10);
+        if (endptr[0]=='\0') menulocal.ignorepath=val;
+      }
+      memfree(f1);
     } else if (strcmp(tok,"save_path")==0) {
       f1=getitok2(&s2,&len," \t,");
       if (f1!=NULL) {
@@ -350,11 +397,17 @@ int mx11loadconfig()
       memfree(menulocal.browser);
       f1=getitok2(&s2,&len,"");
       menulocal.browser=f1;
+    } else if (strcmp(tok,"gpl")==0) {
+      memfree(menulocal.gpl);
+      f1=getitok2(&s2,&len,"");
+      menulocal.gpl=f1;
     } else if (strcmp(tok,"ext_driver")==0) {
       f1=getitok2(&s2,&len,",");
       f2=getitok2(&s2,&len,",");
+      if (s2[1]==',') f3=NULL;
+      else f3=getitok2(&s2,&len,",");
       for (;(s2[0]!='\0') && (strchr(" \t,",s2[0])!=NULL);s2++);
-      f3=getitok2(&s2,&len,"");
+      f4=getitok2(&s2,&len,"");
       if ((f1!=NULL) && (f2!=NULL)) {
         if ((pnew=(struct extprinter *)memalloc(sizeof(struct extprinter)))
         ==NULL) {
@@ -362,6 +415,7 @@ int mx11loadconfig()
           memfree(f1);
           memfree(f2);
           memfree(f3);
+          memfree(f4);
           closeconfig(fp);
           return 1;
         }
@@ -371,11 +425,13 @@ int mx11loadconfig()
         pcur->next=NULL;
         pcur->name=f1;
         pcur->driver=f2;
-        pcur->option=f3;
+        pcur->ext=f3;
+        pcur->option=f4;
       } else {
         memfree(f1);
         memfree(f2);
         memfree(f3);
+        memfree(f4);
       }
     } else if (strcmp(tok,"prn_driver")==0) {
       f1=getitok2(&s2,&len,",");
@@ -438,12 +494,15 @@ int mx11loadconfig()
       f1=getitok2(&s2,&len," \t,");
       f2=getitok2(&s2,&len," \t,");
       f3=getitok2(&s2,&len," \t,");
-      if ((f1!=NULL) && (f2!=NULL) && (f3!=NULL)) {
+      for (;(s2[0]!='\0') && (strchr(" \x09,",s2[0])!=NULL);s2++);
+      f4=getitok2(&s2,&len,"");
+      if ((f1!=NULL) && (f2!=NULL) && (f3!=NULL) && (f4!=NULL)) {
         if ((fnew=memalloc(sizeof(struct fontmap)))==NULL) {
           memfree(tok);
           memfree(f1);
           memfree(f2);
           memfree(f3);
+          memfree(f4);
           closeconfig(fp);
           return 1;
         }
@@ -451,15 +510,21 @@ int mx11loadconfig()
         else fcur->next=fnew;
         fcur=fnew;
         fcur->next=NULL;
-        val=strtol(f2,&endptr,10);
-        memfree(f2);
         fcur->fontalias=f1;
+        if (strcmp(f2,"bold")==0) fcur->type=BOLD;
+        else if (strcmp(f2,"italic")==0) fcur->type=ITALIC;
+        else if (strcmp(f2,"bold_italic")==0) fcur->type=BOLDITALIC;
+        else fcur->type=NORMAL;
+        memfree(f2);
+        val=strtol(f3,&endptr,10);
+        memfree(f3);
         fcur->twobyte=val;
-        fcur->fontname=f3;
+        fcur->fontname=f4;
       } else {
         memfree(f1);
         memfree(f2);
         memfree(f3);
+        memfree(f4);
       }
     } else if (strcmp(tok,"backing_store")==0) {
       f1=getitok2(&s2,&len," \t,");
@@ -507,6 +572,11 @@ int mx11loadconfig()
       val=strtol(f1,&endptr,10);
       if (endptr[0]=='\0') mxlocal->grid=val;
       memfree(f1);
+    } else if (strcmp(tok,"minus_hyphen")==0) {
+      f1=getitok2(&s2,&len," \t,");
+      val=strtol(f1,&endptr,10);
+      if (endptr[0]=='\0') mxlocal->minus_hyphen=val;
+      memfree(f1);
     }
     memfree(tok);
     memfree(str);
@@ -768,10 +838,12 @@ int menuinit(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   menulocal.statusb=TRUE;
   menulocal.movechild=FALSE;
   menulocal.scriptconsole=FALSE;
+  menulocal.addinconsole=TRUE;
   menulocal.mouseclick=500;
   menulocal.changedirectory=1;
   menulocal.editor=NULL;
   menulocal.browser=NULL;
+  menulocal.gpl=NULL;
   menulocal.PaperWidth=21000;
   menulocal.PaperHeight=29700;
   menulocal.LeftMargin=0;
@@ -783,6 +855,10 @@ int menuinit(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   menulocal.exwinbackingstore=FALSE;
   menulocal.fileopendir=NULL;
   menulocal.graphloaddir=NULL;
+  menulocal.expand=1;
+  menulocal.expanddir=(char *)memalloc(3);
+  strcpy(menulocal.expanddir,"./");
+  menulocal.ignorepath=0;
   menulocal.savepath=0;
   menulocal.savewithdata=0;
   menulocal.savewithmerge=0;
@@ -804,6 +880,7 @@ int menuinit(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   mxlocal->grid=200;
   mxlocal->cdepth=X11COLORDEPTH;
   mxlocal->backingstore=FALSE;
+  mxlocal->minus_hyphen=TRUE;
   if (_putobj(obj,"_local",inst,mxlocal)) goto errexit;
   if (mx11loadconfig()) goto errexit; 
   if (exwinloadconfig()) goto errexit;
@@ -838,12 +915,14 @@ int menuinit(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
 errexit:
   memfree(menulocal.editor);
   memfree(menulocal.browser);
+  memfree(menulocal.gpl);
   pcur=menulocal.extprinterroot;
   while (pcur!=NULL) {
     pdel=pcur;
     pcur=pcur->next;
     memfree(pdel->name);
     memfree(pdel->driver);
+    memfree(pdel->ext);
     memfree(pdel->option);
     memfree(pdel);
   }
@@ -893,14 +972,17 @@ int menudone(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   arrayfree2(menulocal.datafilelist);
   memfree(menulocal.editor);
   memfree(menulocal.browser);
+  memfree(menulocal.gpl);
   free(menulocal.fileopendir);
   free(menulocal.graphloaddir);
+  memfree(menulocal.expanddir);
   pcur=menulocal.extprinterroot;
   while (pcur!=NULL) {
     pdel=pcur;
     pcur=pcur->next;
     memfree(pdel->name);
     memfree(pdel->driver);
+    memfree(pdel->ext);
     memfree(pdel->option);
     memfree(pdel);
   }
@@ -1008,7 +1090,7 @@ int mxredrawflag(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
 void mx_redraw(struct objlist *obj,char *inst)
 {
   GRAredraw(obj,inst,TRUE,mxlocal->redrawf);
-  if (Disp!=NULL) XFlush(Disp);
+  mxflush(obj,inst,NULL,0,NULL);
 }
 
 void mx_inslist(struct objlist *obj,char *inst,
@@ -1137,22 +1219,22 @@ void mxrestoreGC(struct mxlocal *mxsave)
 
 int mxd2p(int r)
 {
-  return round(r*mxlocal->pixel_dot);
+  return nround(r*mxlocal->pixel_dot);
 }
 
 int mxd2px(int x)
 {
-  return round(x*mxlocal->pixel_dot+mxlocal->offsetx-mxlocal->scrollx);
+  return nround(x*mxlocal->pixel_dot+mxlocal->offsetx-mxlocal->scrollx);
 }
 
 int mxd2py(int y)
 {
-  return round(y*mxlocal->pixel_dot+mxlocal->offsety-mxlocal->scrolly);
+  return nround(y*mxlocal->pixel_dot+mxlocal->offsety-mxlocal->scrolly);
 }
 
 int mxp2d(int r)
 {
-  return round(r/mxlocal->pixel_dot);
+  return nround(r/mxlocal->pixel_dot);
 }
 
 unsigned long RGB(int R,int G,int B)
@@ -1185,12 +1267,12 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
   char **fname2,**fname3;
   struct fontlocal font;
   struct fontmap *fcur;
-  char fontmatrix[128];
+  char fontmatrix[256];
   int rcount2,rcount3;
   Font newfont;
   XFontStruct *newfontstruct;
   int store;
-  int twobyte=FALSE;
+  int twobyte=FALSE,type;
 
   fontcashfind=-1;
   for (i=0;i<mxlocal->loadfont;i++) {
@@ -1216,6 +1298,7 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
   while (fcur!=NULL) {
     if (strcmp(fontalias,fcur->fontalias)==0) {
       fontname=fcur->fontname;
+      type=fcur->type;
       twobyte=fcur->twobyte;
       break;
     }
@@ -1236,8 +1319,14 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
   strncpy(fontname3,fontname,fsp-fontname+1);
   fontname2[fsp-fontname+1]='\0';
   fontname3[fsp-fontname+1]='\0';
-  sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
-          size*fcos,size*fsin,-size*fsin,size*fcos);
+  if ((type==ITALIC) || (type==BOLDITALIC)) {
+    sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
+            size*fcos,size*fsin,
+           -size*fsin+size*fcos*0.36,size*fcos+size*fsin*0.36);
+  } else {
+    sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
+            size*fcos,size*fsin,-size*fsin,size*fcos);
+  }
   for (i=0;fontmatrix[i]!='\0';i++)
     if (fontmatrix[i]=='-') fontmatrix[i]='~';
   strcat(fontname2,fontmatrix);
@@ -1278,7 +1367,12 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
     (mxlocal->font[store]).font=newfont;
     (mxlocal->font[store]).fontstruct=newfontstruct;
     (mxlocal->font[store]).fontsize=size;
+    (mxlocal->font[store]).fonttype=type;
     (mxlocal->font[store]).fontdir=dir;
+    if (strcmp(fname2[0]+strlen(fname2[0])-9,"iso8859-1")==0)
+      (mxlocal->font[store]).iso8859=TRUE;
+    else
+      (mxlocal->font[store]).iso8859=FALSE;
     if (top) XSetFont(disp,gc,(mxlocal->font[store]).font);
     mxlocal->loadfont++;
     if (fname2!=NULL) XFreeFontNames(fname2);
@@ -1296,6 +1390,7 @@ int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   int *cpar;
   char *cstr,ch[1];
   int i,x,y,x1,y1,x2,y2;
+  double bs;
   XRectangle rect;
   int width,style,cap,join,dashn=0;
   char *dashlist=NULL;
@@ -1305,7 +1400,7 @@ int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
 
   double fontsize,fontspace,fontdir,fontsin,fontcos;
   int fontcashsize,fontcashdir;
-  double x0,y0,fontwidth;
+  double x0,y0,x02,y02,fontwidth;
   XChar2b kanji[1];
   unsigned int jis;
 
@@ -1500,6 +1595,7 @@ int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
     fontspace=cpar[2]/72.0*25.4;
     mxlocal->fontspace=fontspace;
     fontsize=cpar[1]/72.0*25.4;
+    mxlocal->fontsize=fontsize;
     fontcashsize=mxd2p(fontsize);
     fontcashdir=cpar[3];
     fontdir=cpar[3]*MPI/18000.0;
@@ -1533,18 +1629,35 @@ int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         i++;
       }
       XDrawString(Disp,mxlocal->win,mxlocal->gc,
-                  mxd2px(round(x0)),
-                  mxd2py(round(y0)),ch,1); 
+                  mxd2px(nround(x0)),
+                  mxd2py(nround(y0)),ch,1); 
+      if ((mxlocal->font[0].fonttype==BOLD) 
+       || (mxlocal->font[0].fonttype==BOLDITALIC)) {
+        bs=mxlocal->fontsize*0.018;
+        XDrawString(Disp,mxlocal->win,mxlocal->gc,
+          mxd2px(nround(x0+bs)),
+          mxd2py(nround(y0)),ch,1); 
+        XDrawString(Disp,mxlocal->win,mxlocal->gc,
+          mxd2px(nround(x0)),
+          mxd2py(nround(y0-bs)),ch,1); 
+        XDrawString(Disp,mxlocal->win,mxlocal->gc,
+          mxd2px(nround(x0+bs)),
+          mxd2py(nround(y0-bs)),ch,1); 
+      }
       if ((mxlocal->font[0]).fontstruct==NULL) {
         fontwidth=mxp2d((mxlocal->font[0]).fontsize);
       } else {
+        if ((mxlocal->font[0]).iso8859
+        && mxlocal->minus_hyphen && (ch[0]=='-')) {
+          ch[0]='+';
+        }
         fontwidth=mxp2d(XTextWidth((mxlocal->font[0]).fontstruct,ch,1));
       }
       x0+=(fontwidth+mxlocal->fontspace)*mxlocal->fontcos;
       y0-=(fontwidth+mxlocal->fontspace)*mxlocal->fontsin; 
     }
-    mxlocal->cpx=round(x0);
-    mxlocal->cpy=round(y0);
+    mxlocal->cpx=nround(x0);
+    mxlocal->cpy=nround(y0);
     break;
   case 'K':
     if (mxlocal->loadfontf==-1) break;
@@ -1557,8 +1670,21 @@ int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         kanji[0].byte1=jis >> 8;
         kanji[0].byte2=jis & 0xff;
         XDrawString16(Disp,mxlocal->win,mxlocal->gc,
-                  mxd2px(round(x0)),
-                  mxd2py(round(y0)),kanji,1);
+                  mxd2px(nround(x0)),
+                  mxd2py(nround(y0)),kanji,1);
+        if ((mxlocal->font[0].fonttype==BOLD) 
+         || (mxlocal->font[0].fonttype==BOLDITALIC)) {
+          bs=mxlocal->fontsize*0.018;
+          XDrawString16(Disp,mxlocal->win,mxlocal->gc,
+            mxd2px(nround(x0+bs)),
+            mxd2py(nround(y0)),kanji,1); 
+          XDrawString16(Disp,mxlocal->win,mxlocal->gc,
+            mxd2px(nround(x0)),
+            mxd2py(nround(y0-bs)),kanji,1); 
+          XDrawString16(Disp,mxlocal->win,mxlocal->gc,
+            mxd2px(nround(x0+bs)),
+            mxd2py(nround(y0-bs)),kanji,1); 
+        }
         if ((mxlocal->font[0]).fontstruct==NULL) {
           fontwidth=mxp2d((mxlocal->font[0]).fontsize);
         } else {
@@ -1569,8 +1695,8 @@ int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         i+=2;
       } else i++;
     }
-    mxlocal->cpx=round(x0);
-    mxlocal->cpy=round(y0);
+    mxlocal->cpx=nround(x0);
+    mxlocal->cpy=nround(y0);
     break;
   default: break;
   }
@@ -1601,7 +1727,7 @@ int mx_charwidth(struct objlist *obj,char *inst,char *rval,
     fontstruct=(mxlocal->font[cashpos]).fontstruct;    
     if (fontstruct==NULL) {
       if (ch[1]!=0) *(int *)rval=size;
-      else *(int *)rval=round(size*0.600);
+      else *(int *)rval=nround(size*0.600);
     } else {
       if (ch[1]!=0) {
         jis=njms2jis(*(unsigned int *)(argv[3]));
@@ -1609,10 +1735,14 @@ int mx_charwidth(struct objlist *obj,char *inst,char *rval,
         kanji[0].byte2=jis & 0xff;
         *(int *)rval=mxp2d(XTextWidth16(fontstruct,kanji,1));
       } else {
+        if ((mxlocal->font[cashpos]).iso8859 && mxlocal->minus_hyphen
+        && (ch[0]=='-')) {
+          ch[0]='+';
+        }
         *(int *)rval=mxp2d(XTextWidth(fontstruct,ch,1));
       }
     }
-  } else *(int *)rval=round(size*0.600);
+  } else *(int *)rval=nround(size*0.600);
   return 0;
 }
 
@@ -1650,19 +1780,19 @@ int mx_charheight(struct objlist *obj,char *inst,char *rval,
     fontstruct=(mxlocal->font[cashpos]).fontstruct;    
     if (fontstruct==NULL) {
       if (twobyte) {
-        if (height) *(int *)rval=round(size*0.791);
-        else *(int *)rval=round(size*0.250);
+        if (height) *(int *)rval=nround(size*0.791);
+        else *(int *)rval=nround(size*0.250);
       } else {
-        if (height) *(int *)rval=round(size*0.562);
-        else *(int *)rval=round(size*0.250);
+        if (height) *(int *)rval=nround(size*0.562);
+        else *(int *)rval=nround(size*0.250);
       }
     } else {
       if (height) *(int *)rval=mxp2d(fontstruct->ascent);
       else *(int *)rval=mxp2d(fontstruct->descent);
     }
   } else {
-    if (height) *(int *)rval=round(size*0.562);
-    else *(int *)rval=round(size*0.250);
+    if (height) *(int *)rval=nround(size*0.562);
+    else *(int *)rval=nround(size*0.250);
   }
   return 0;
 }
