@@ -1,21 +1,35 @@
-/**
- *
- * $Id: x11commn.c,v 1.4 1999/04/15 12:14:26 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11commn.c,v 1.6 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11commn.c,v $
+ * Revision 1.6  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.5  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
  * Revision 1.4  1999/04/15 12:14:26  isizaka
  * for release 6.03.01
  *
@@ -1362,23 +1376,10 @@ void ExtTextOut(Display *disp,Window win,GC gc,
 {
   Region region;
   XRectangle rect2;
-  int i;
-  unsigned int jis;
 
   region=XCreateRegion();
   XUnionRectWithRegion(rect,region,region);
   XSetRegion(disp,gc,region);
-#ifdef JAPANESE
-/* SJIS ---> EUC */
-  for (i=0;i<strlen(buf);i++) {
-    if (niskanji((unsigned char)buf[i])) {
-      jis=njms2jis(((unsigned char)buf[i] << 8)+(unsigned char)buf[i+1]);
-      buf[i]=(jis >> 8) | 0x80;
-      buf[i+1]=(jis & 0xff) | 0x80; 
-      i++;
-    }
-  }
-#endif
   XmbDrawString(disp,win,FFont,gc,x,y,buf,len);
   XDestroyRegion(region);
   rect2.x=0;
