@@ -1,21 +1,35 @@
-/**
- *
- * $Id: x11dialg.c,v 1.3 1999/04/11 06:09:20 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11dialg.c,v 1.5 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11dialg.c,v $
+ * Revision 1.5  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.4  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
  * Revision 1.3  1999/04/11 06:09:20  isizaka
  * *** empty log message ***
  *
@@ -107,6 +121,7 @@ struct SaveDialog DlgSave;
 struct AboutDialog DlgAbout;
 struct DriverDialog DlgDriver;
 struct PrintDialog DlgPrinter;
+struct OutputDataDialog DlgOutputData;
 struct ScriptDialog DlgScript;
 struct DefaultDialog DlgDefault;
 struct SetScriptDialog DlgSetScript;
@@ -209,6 +224,8 @@ void initdialog()
   DlgDriver.resource="driver";
   DlgPrinter.widget=NULL;
   DlgPrinter.resource="print";
+  DlgOutputData.widget=NULL;
+  DlgOutputData.resource="outputdata";
   DlgScript.widget=NULL;
   DlgScript.resource="addin";
   DlgDefault.widget=NULL;
@@ -449,6 +466,21 @@ void SetTextFromObjField(Widget w,char *res,struct objlist *Obj,int Id,
   memfree(buf);
 }
 
+void SetComboListFromObjField(Widget w,char *res,struct objlist *Obj,int Id,
+                              char *field)
+{
+  char *buf;
+  XmString xs;
+
+  sgetobjfield(Obj,Id,field,NULL,&buf,FALSE,FALSE,FALSE);
+  if (buf!=NULL) {
+    xs=XmStringCreateLocalized(buf);
+    XmListSelectItem(w,xs,FALSE);
+    XmStringFree(xs);
+    memfree(buf);
+  }
+}
+
 int SetObjFieldFromToggle(Widget w,char *res,struct objlist *Obj,int Id,
                           char *field)
 {
@@ -504,12 +536,17 @@ void SetStyleFromObjField(Widget w,char *res,struct objlist *Obj,int Id,
   int *style,a;
   XmString xs;
   char *s;
+  int count;
 
-  XmListDeleteAllItems(GetComboBoxList(w,res));
-  for (j=0;j<CLINESTYLE;j++) {
-    xs=XmStringCreateLocalized(fwlinestyle[j]);
-    XmListAddItem(GetComboBoxList(w,res),xs,0);
-    XmStringFree(xs);
+  XtVaGetValues(GetComboBoxList(w,res),XmNitemCount,&count,NULL);
+  if (count==0) {
+    XmListDeleteAllItems(GetComboBoxList(w,res));
+    for (j=0;j<CLINESTYLE;j++) {
+      xs=XmStringCreateLocalized(fwlinestyle[j]);
+      XmListAddItem(GetComboBoxList(w,res),xs,0);
+      XmStringFree(xs);
+    }
+    SetComboBoxVisibleItemCount(w,res,j);
   }
   getobj(Obj,field,Id,0,NULL,&array);
   stylenum=arraynum(array);
@@ -550,19 +587,43 @@ void SetListFromObjField(Widget w,char *res,struct objlist *Obj,int Id,
   char **enumlist;
   int j,a;
   XmString xs;
+  int count;
 
-  XmListDeleteAllItems(GetComboBoxList(w,res));
-  enumlist=(char **)chkobjarglist(Obj,field);
-  for (j=0;enumlist[j]!=NULL;j++) {
-    xs=XmStringCreateLocalized(enumlist[j]);
-    XmListAddItem(GetComboBoxList(w,res),xs,0);
-    XmStringFree(xs);
+  XtVaGetValues(GetComboBoxList(w,res),XmNitemCount,&count,NULL);
+  if (count==0) {
+    XmListDeleteAllItems(GetComboBoxList(w,res));
+    enumlist=(char **)chkobjarglist(Obj,field);
+    for (j=0;enumlist[j]!=NULL;j++) {
+      xs=XmStringCreateLocalized(enumlist[j]);
+      XmListAddItem(GetComboBoxList(w,res),xs,0);
+      XmStringFree(xs);
+    }
+    SetComboBoxVisibleItemCount(w,res,j);
   }
   getobj(Obj,field,Id,0,NULL,&a);
   XmListSelectPos(GetComboBoxList(w,res),a+1,TRUE);
 }
 
 void SetComboList(Widget w,char *res,char **list,int num)
+{
+  int j;
+  XmString xs;
+  int count;
+
+  XtVaGetValues(GetComboBoxList(w,res),XmNitemCount,&count,NULL);
+  if (count==0) {
+    XmListDeleteAllItems(GetComboBoxList(w,res));
+    for (j=0;j<num;j++) {
+      xs=XmStringCreateLocalized(list[j]);
+      XmListAddItem(GetComboBoxList(w,res),xs,0);
+      XmStringFree(xs);
+    }
+    SetComboBoxVisibleItemCount(w,res,j);
+  }
+  XmListDeselectAllItems(GetComboBoxList(w,res));
+}
+
+void SetComboList2(Widget w,char *res,char **list,int num)
 {
   int j;
   XmString xs;
@@ -573,6 +634,8 @@ void SetComboList(Widget w,char *res,char **list,int num)
     XmListAddItem(GetComboBoxList(w,res),xs,0);
     XmStringFree(xs);
   }
+  SetComboBoxVisibleItemCount(w,res,j);
+  XmListDeselectAllItems(GetComboBoxList(w,res));
 }
 
 Widget GetComboBoxText(Widget w,char *res)
@@ -586,3 +649,9 @@ Widget GetComboBoxList(Widget w,char *res)
   if (res==NULL) return XtNameToWidget(w,COMBOLIST);
   else return XtNameToWidget(XtNameToWidget(w,res),COMBOLIST);
 }
+
+void SetComboBoxVisibleItemCount(Widget w,char *res,int count)
+{
+  if ((count<=0) || (count>30)) count=10;
+  XtVaSetValues(GetComboBoxList(w,res),XmNvisibleItemCount,count-1,NULL);
+}
