@@ -1,21 +1,38 @@
-/**
- *
- * $Id: x11dialg.h,v 1.3 1999/04/11 06:09:20 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11dialg.h,v 1.6 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11dialg.h,v $
+ * Revision 1.6  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.5  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.4  2001/07/14 17:44:50  isizaka
+ * for 6.03.14
+ *
  * Revision 1.3  1999/04/11 06:09:20  isizaka
  * *** empty log message ***
  *
@@ -49,6 +66,7 @@ int CopyClick(Widget parent,struct objlist *obj,int Id,
               char *(*callback)(struct objlist *obj,int id));
 int SetObjFieldFromText(Widget w,char *res,struct objlist *Obj,int Id,char *field);
 void SetTextFromObjField(Widget w,char *res,struct objlist *Obj,int Id,char *field);
+void SetComboListFromObjField(Widget w,char *res,struct objlist *Obj,int Id,char *field);
 int SetObjFieldFromToggle(Widget w,char *res,struct objlist *Obj,int Id,char *field);
 void SetToggleFromObjField(Widget w,char *res,struct objlist *Obj,int Id,char *field);
 int SetObjFieldFromStyle(Widget w,char *res,struct objlist *Obj,int Id,char *field);
@@ -56,8 +74,10 @@ void SetStyleFromObjField(Widget w,char *res,struct objlist *Obj,int Id,char *fi
 int SetObjFieldFromList(Widget w,char *res,struct objlist *Obj,int Id,char *field);
 void SetListFromObjField(Widget w,char *res,struct objlist *Obj,int Id,char *field);
 void SetComboList(Widget w,char *res,char **list,int num);
+void SetComboList2(Widget w,char *res,char **list,int num);
 Widget GetComboBoxList(Widget w,char *res);
 Widget GetComboBoxText(Widget w,char *res);
+void SetComboBoxVisibleItemCount(Widget w,char *res,int count);
 
 struct MarkDialog {
   Widget parent,widget;
@@ -386,7 +406,7 @@ struct LegendArrowDialog {
   struct objlist *Obj;
   int Id;
   int R,G,B;
-  int wid,len;
+  int wid,ang;
 };
 void LegendArrowDialog(struct LegendArrowDialog *data,
                        struct objlist *obj,int id);
@@ -561,6 +581,16 @@ struct PrintDialog {
 void PrintDialog(struct PrintDialog *data,
                  struct objlist *obj,int id);
 
+struct OutputDataDialog {
+  Widget parent,widget;
+  int ret;
+  char *resource;
+  void (*SetupWindow)(Widget w,void *data,int makewidget);
+  void (*CloseWindow)(Widget w,void *data);
+  int div;
+};
+void OutputDataDialog(struct OutputDataDialog *data,int div);
+
 struct ScriptDialog {
   Widget parent,widget;
   int ret;
@@ -727,6 +757,7 @@ extern struct SaveDialog DlgSave;
 extern struct AboutDialog DlgAbout;
 extern struct DriverDialog DlgDriver;
 extern struct PrintDialog DlgPrinter;
+extern struct OutputDataDialog DlgOutputData;
 extern struct ScriptDialog DlgScript;
 extern struct DefaultDialog DlgDefault;
 extern struct SetScriptDialog DlgSetScript;
