@@ -1,21 +1,41 @@
-/**
- *
- * $Id: x11graph.c,v 1.4 1999/05/08 13:31:30 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11graph.c,v 1.8 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11graph.c,v $
+ * Revision 1.8  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.7  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.6  2001/07/14 17:44:50  isizaka
+ * for 6.03.14
+ *
+ * Revision 1.5  2001/03/23 12:17:43  isizaka
+ * for 6.3.13
+ *
  * Revision 1.4  1999/05/08 13:31:30  isizaka
  * for release 6.03.02
  *
@@ -159,6 +179,7 @@ void PageDialogSetup(Widget w,void *data,int makewidget)
     XmListAddItem(GetComboBoxList(w,"*paper"),xs,0);
     XmStringFree(xs);
   } 
+  SetComboBoxVisibleItemCount(w,"*paper",j);
   XtAddCallback(cmb,XmNselectionCallback,PageDialogPage,d);
   ac=0;
   XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
@@ -372,7 +393,6 @@ void SwitchDialogSetup(Widget w,void *data,int makewidget)
   ac=0;
   XtManageChild(XmCreateLabel(rc2,"DrawOrder",al,ac));
   ac=0;
-  XtSetArg(al[ac],XmNvisibleItemCount,10); ac++;
   XtSetArg(al[ac],XmNlistSizePolicy,XmCONSTANT); ac++;
   XtSetArg(al[ac],XmNscrollBarDisplayPolicy,XmSTATIC); ac++;
   XtSetArg(al[ac],XmNselectionPolicy,XmEXTENDED_SELECT); ac++;
@@ -404,7 +424,6 @@ void SwitchDialogSetup(Widget w,void *data,int makewidget)
   ac=0;
   XtManageChild(XmCreateLabel(rc2,"Objects",al,ac));
   ac=0;
-  XtSetArg(al[ac],XmNvisibleItemCount,10); ac++;
   XtSetArg(al[ac],XmNlistSizePolicy,XmCONSTANT); ac++;
   XtSetArg(al[ac],XmNscrollBarDisplayPolicy,XmSTATIC); ac++;
   XtSetArg(al[ac],XmNselectionPolicy,XmEXTENDED_SELECT); ac++;
@@ -527,7 +546,7 @@ void LoadDialogSetup(Widget w,void *data,int makewidget)
   XtVaSetValues(XtNameToWidget(w,"*ExpandIncludedFile"),
                 XmNset,d->expand,NULL);
   XtVaSetValues(XtNameToWidget(w,"*dir"),XmNvalue,d->exdir,NULL);
-  XtVaSetValues(XtNameToWidget(w,"*IgnoreFilePath"),XmNset,FALSE,NULL);
+  XtVaSetValues(XtNameToWidget(w,"*IgnoreFilePath"),XmNset,d->ignorepath,NULL);
 }
 
 void LoadDialogClose(Widget w,void *data)
@@ -549,10 +568,10 @@ void LoadDialog(struct LoadDialog *data)
 {
   data->SetupWindow=LoadDialogSetup;
   data->CloseWindow=LoadDialogClose;
-  data->expand=TRUE;
-  data->exdir=memalloc(3);
-  strcpy(data->exdir,"./");
-  data->ignorepath=FALSE;
+  data->expand=menulocal.expand;
+  data->exdir=memalloc(strlen(menulocal.expanddir)+1);
+  strcpy(data->exdir,menulocal.expanddir);
+  data->ignorepath=menulocal.ignorepath;
 }
 
 void PrmDialogSetup(Widget w,void *data,int makewidget)
@@ -611,8 +630,6 @@ void SaveDialogSetup(Widget w,void *data,int makewidget)
   Arg al[20];
   Cardinal ac;
   Widget rc,rc1;
-  XmString s;
-  char **enumlist;
   int j;
   struct SaveDialog *d;
   
@@ -634,12 +651,9 @@ void SaveDialogSetup(Widget w,void *data,int makewidget)
     ac=0;
     XtManageChild(XmCreateToggleButton(rc,"IncludeMergeFile",al,ac));
 
-    enumlist=pathchar;
-    for (j=0;enumlist[j]!=NULL;j++) {
-      s=XmStringCreateLocalized(enumlist[j]);
-      XmListAddItem(GetComboBoxList(w,"*path"),s,0);
-      XmStringFree(s);
-    }
+    for (j=0;pathchar[j]!=NULL;j++);
+    SetComboList(w,"*path",pathchar,j);
+
   }
   XmListDeselectAllItems(GetComboBoxList(w,"*path"));
   XmListSelectPos(GetComboBoxList(w,"*path"),menulocal.savepath+1,TRUE);
@@ -924,13 +938,25 @@ void CmGraphMenu(Widget w,XtPointer client_data,XtPointer call_data)
   }
 }
 
+void AboutGPL(Widget w,XtPointer client_data,XtPointer call_data)
+{
+  pid_t pid;
+
+  if (menulocal.gpl==NULL) return;
+  if ((pid=fork())<0) return;
+  if (pid==0) {
+    system(menulocal.gpl);
+    exit(0);
+  }
+}
+
 void AboutDialogSetup(Widget w,void *data,int makewidget)
 {
   Arg al[20];
   Cardinal ac;
-  Widget rc,label;
+  Widget button,rc0,rc,label;
   struct objlist *obj;
-  char *version,*email,*copyright;
+  char *version,*email,*web,*copyright;
   char buf[512];
   XmString msg;
 
@@ -939,9 +965,16 @@ void AboutDialogSetup(Widget w,void *data,int makewidget)
     getobj(obj,"version",0,0,NULL,&version);
     getobj(obj,"copyright",0,0,NULL,&copyright);
     getobj(obj,"e-mail",0,0,NULL,&email);
+    getobj(obj,"web",0,0,NULL,&web);
+    ac=0;
+    XtManageChild(button=XmCreatePushButton(w,"GPL",al,ac));
+    XtAddCallback(button,XmNdisarmCallback,AboutGPL,NULL);
+    ac=0;
+    XtSetArg(al[ac],XmNorientation,XmVERTICAL); ac++;
+    XtManageChild(rc0=XmCreateRowColumn(w,NULL,al,ac));
     ac=0;
     XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
-    XtManageChild(rc=XmCreateRowColumn(w,NULL,al,ac));
+    XtManageChild(rc=XmCreateRowColumn(rc0,NULL,al,ac));
     ac=0;
     XtSetArg(al[ac],XmNlabelType,XmPIXMAP); ac++;
     XtSetArg(al[ac],XmNlabelPixmap,NgraphApp.iconpix); ac++;
@@ -951,13 +984,23 @@ void AboutDialogSetup(Widget w,void *data,int makewidget)
     sprintf(buf,
     "Ngraph "PLATFORM
     "\nversion %s"GUIVERSION
-    "\n\n%s\nAll rights reserved\n%s",version,copyright,email);
+    "\n\n%s\n%s\n%s\n",version,copyright,email,web);
     msg=XmStringCreateLtoR(buf,XmFONTLIST_DEFAULT_TAG);
     ac=0;
     XtSetArg(al[ac],XmNlabelString,msg); ac++;
     XtManageChild(label=XmCreateLabel(rc,NULL,al,ac));
     XtVaSetValues(label,XmNalignment,XmALIGNMENT_CENTER,NULL);
     XmStringFree(msg);
+
+    sprintf(buf,
+"Ngraph comes with ABSOLUTELY NO WARRANTY; for details \npush [GPL] below. This is free software, and you are welcome \nto redistribute it under certain conditions; push [GPL] below \nfor details.");
+    msg=XmStringCreateLtoR(buf,XmFONTLIST_DEFAULT_TAG);
+    ac=0;
+    XtSetArg(al[ac],XmNlabelString,msg); ac++;
+    XtManageChild(label=XmCreateLabel(rc0,NULL,al,ac));
+    XtVaSetValues(label,XmNalignment,XmALIGNMENT_BEGINNING,NULL);
+    XmStringFree(msg);
+
     XtUnmanageChild(XmMessageBoxGetChild(w,XmDIALOG_CANCEL_BUTTON));
   }
 }
@@ -976,7 +1019,7 @@ void CmHelpAbout()
 
 void CmHelpHelp()
 {
-  int pid;
+  pid_t pid;
 
   if (menulocal.browser==NULL) return;
   if ((pid=fork())<0) return;
