@@ -1,21 +1,41 @@
-/**
- *
- * $Id: x11lgnd.c,v 1.6 1999/07/25 11:28:13 isizaka Exp isizaka $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11lgnd.c,v 1.10 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11lgnd.c,v $
+ * Revision 1.10  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.9  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.8  2001/07/14 17:44:50  isizaka
+ * for 6.03.14
+ *
+ * Revision 1.7  2001/03/23 12:17:43  isizaka
+ * for 6.3.13
+ *
  * Revision 1.6  1999/07/25 11:28:13  isizaka
  * for release 6.03.04
  *
@@ -110,8 +130,14 @@ char *LegendTextCB(struct objlist *obj,int id)
 
   if ((s=(char *)memalloc(128))==NULL) return NULL;
   getobj(obj,"text",id,0,NULL,&text);
-  if (text==NULL) sprintf(s,"%-5d",id);
-  else sprintf(s,"%-5d %.64s",id,text);
+  sprintf(s,"%-5d",id);
+  if (text!=NULL) {
+    strncat(s, text, 64);
+#ifdef JAPANESE
+/* SJIS ---> EUC */
+    njms2euc(s);
+#endif
+  }
   return s;
 }
 
@@ -121,6 +147,7 @@ void LegendCurveDialogSetupItem(Widget w,struct LegendCurveDialog *d,int id)
   SetStyleFromObjField(w,"*style",d->Obj,id,"style");
   SetComboList(w,"*width",cblinewidth,CBLINEWIDTH);
   SetTextFromObjField(GetComboBoxText(w,"*width"),NULL,d->Obj,id,"width");
+  SetComboListFromObjField(GetComboBoxList(w,"*width"),NULL,d->Obj,id,"width");
   SetListFromObjField(w,"*join",d->Obj,id,"join");
   SetTextFromObjField(w,"*miter",d->Obj,id,"miter_limit");
   getobj(d->Obj,"R",id,0,NULL,&d->R);
@@ -273,6 +300,7 @@ void LegendPolyDialogSetupItem(Widget w,struct LegendPolyDialog *d,int id)
   SetStyleFromObjField(w,"*style",d->Obj,id,"style");
   SetComboList(w,"*width",cblinewidth,CBLINEWIDTH);
   SetTextFromObjField(GetComboBoxText(w,"*width"),NULL,d->Obj,id,"width");
+  SetComboListFromObjField(GetComboBoxList(w,"*width"),NULL,d->Obj,id,"width");
   SetListFromObjField(w,"*join",d->Obj,id,"join");
   SetTextFromObjField(w,"*miter",d->Obj,id,"miter_limit");
   getobj(d->Obj,"R",id,0,NULL,&d->R);
@@ -419,28 +447,29 @@ void LegendPolyDialog(struct LegendPolyDialog *data,struct objlist *obj,int id)
 
 void LegendArrowDialogSetupItem(Widget w,struct LegendArrowDialog *d,int id)
 {
-  char buf[30];
-  XmString xs;
+  int len;
 
   SetTextFromObjField(w,"*points",d->Obj,id,"points");
   SetStyleFromObjField(w,"*style",d->Obj,id,"style");
   SetComboList(w,"*width",cblinewidth,CBLINEWIDTH);
   SetTextFromObjField(GetComboBoxText(w,"*width"),NULL,d->Obj,id,"width");
+  SetComboListFromObjField(GetComboBoxList(w,"*width"),NULL,d->Obj,id,"width");
   SetListFromObjField(w,"*join",d->Obj,id,"join");
   SetTextFromObjField(w,"*miter",d->Obj,id,"miter_limit");
   SetListFromObjField(w,"*arrow",d->Obj,id,"arrow");
-  getobj(d->Obj,"arrow_length",id,0,NULL,&(d->len));
+  getobj(d->Obj,"arrow_length",id,0,NULL,&len);
   getobj(d->Obj,"arrow_width",id,0,NULL,&(d->wid));
-  XtVaSetValues(XtNameToWidget(w,"*scl"),XmNvalue,d->len,NULL);
-  XtVaSetValues(XtNameToWidget(w,"*scw"),XmNvalue,d->wid,NULL);
-  sprintf(buf,"Head angle: %.1f degree",atan(0.5*d->wid/d->len)*2*180/MPI);
-  xs=XmStringCreateLocalized(buf);
-  XtVaSetValues(XtNameToWidget(w,"*head"),XmNlabelString,xs,NULL);
-  XmStringFree(xs);
+  d->wid=(d->wid/100)*100;
+  d->ang=atan(0.5*d->wid/len)*2*180/MPI;
+  if (d->ang<10) d->ang=10;
+  else if (d->ang>170) d->ang=170;
+  XtVaSetValues(XtNameToWidget(w,"*sca"),XmNvalue,d->ang,NULL);
+  XtVaSetValues(XtNameToWidget(w,"*scw"),XmNvalue,d->wid/100,NULL);
   getobj(d->Obj,"R",id,0,NULL,&d->R);
   getobj(d->Obj,"G",id,0,NULL,&d->G);
   getobj(d->Obj,"B",id,0,NULL,&d->B);
-  XtVaSetValues(XtNameToWidget(w,"*col"),XmNbackground,RGB(d->R,d->G,d->B),NULL);
+  XtVaSetValues(XtNameToWidget(w,"*col"),XmNbackground,RGB(d->R,d->G,d->B),
+                NULL);
 }
 
 void LegendArrowDialogPaint(Widget w,XtPointer client_data,XtPointer call_data)
@@ -449,7 +478,7 @@ void LegendArrowDialogPaint(Widget w,XtPointer client_data,XtPointer call_data)
   XmDrawingAreaCallbackStruct *dd;
   Window root,win;
   Display *disp;
-  int x,y,lw;
+  int x,y,lw,len;
   unsigned int ww,hh,border,depth;
   GC gc;
   XPoint points[3];
@@ -466,13 +495,14 @@ void LegendArrowDialogPaint(Widget w,XtPointer client_data,XtPointer call_data)
   XSetForeground(disp,gc,BlackPixel(XtDisplay(TopLevel),0));
   if (ww/20<hh/20) lw=ww/20;
   else lw=hh/20;
-  XFillRectangle(disp,win,gc,x+round(lw*(d->len/10000.0)),y+(hh-lw)/2,ww,lw);
+  len=d->wid*0.5/tan(d->ang*0.5*MPI/180);
+  XFillRectangle(disp,win,gc,x+nround(lw*(len/10000.0)),y+(hh-lw)/2,ww,lw);
   points[0].x=x;
   points[0].y=y+hh/2;
-  points[1].x=x+round(lw*(d->len/10000.0));
-  points[1].y=y+hh/2-round(lw*(d->wid/20000.0));
-  points[2].x=x+round(lw*(d->len/10000.0));
-  points[2].y=y+hh/2+round(lw*(d->wid/20000.0));
+  points[1].x=x+nround(lw*(len/10000.0));
+  points[1].y=y+hh/2-nround(lw*(d->wid/20000.0));
+  points[2].x=x+nround(lw*(len/10000.0));
+  points[2].y=y+hh/2+nround(lw*(d->wid/20000.0));
   XFillPolygon(disp,win,gc,points,3,Complex,CoordModeOrigin);
   XFreeGC(disp,gc);
 }
@@ -482,18 +512,12 @@ void LegendArrowDialogScaleW(Widget w,XtPointer client_data,XtPointer call_data)
   struct LegendArrowDialog *d;
   XmScaleCallbackStruct *dd;
   Widget win;
-  char buf[30];
-  XmString xs;
 
   d=(struct LegendArrowDialog *)client_data;
   dd=(XmScaleCallbackStruct *)call_data;
-  d->wid=dd->value;
+  d->wid=dd->value*100;
   win=XtNameToWidget(d->widget,"*win");
   XClearArea(XtDisplay(win),XtWindow(win),0,0,0,0,TRUE); 
-  sprintf(buf,"Head angle: %.1f degree",atan(0.5*d->wid/d->len)*2*180/MPI);
-  xs=XmStringCreateLocalized(buf);
-  XtVaSetValues(XtNameToWidget(d->widget,"*head"),XmNlabelString,xs,NULL);
-  XmStringFree(xs);
 }
 
 void LegendArrowDialogScaleL(Widget w,XtPointer client_data,XtPointer call_data)
@@ -506,13 +530,9 @@ void LegendArrowDialogScaleL(Widget w,XtPointer client_data,XtPointer call_data)
 
   d=(struct LegendArrowDialog *)client_data;
   dd=(XmScaleCallbackStruct *)call_data;
-  d->len=dd->value;
+  d->ang=dd->value;
   win=XtNameToWidget(d->widget,"*win");
   XClearArea(XtDisplay(win),XtWindow(win),0,0,0,0,TRUE); 
-  sprintf(buf,"Head angle: %.1f degree",atan(0.5*d->wid/d->len)*2*180/MPI);
-  xs=XmStringCreateLocalized(buf);
-  XtVaSetValues(XtNameToWidget(d->widget,"*head"),XmNlabelString,xs,NULL);
-  XmStringFree(xs);
 }
 
 void LegendArrowDialogDelete(Widget w,XtPointer client_data,XtPointer call_data)
@@ -546,7 +566,7 @@ void LegendArrowDialogSetup(Widget w,void *data,int makewidget)
 {
   Arg al[20];
   Cardinal ac;
-  Widget rc0,rc,rc2,button,da,sc,lb;
+  Widget rc0,rc,rc2,button,da,sc;
   struct LegendArrowDialog *d;
   char title[30];
   XmString xs;
@@ -637,25 +657,15 @@ void LegendArrowDialogSetup(Widget w,void *data,int makewidget)
   XtSetArg(al[ac],XmNcomboBoxType,XmDROP_DOWN_COMBO_BOX); ac++;
   XtManageChild(XmCreateComboBox(rc2,"arrow",al,ac));
 
-  ac=0;
-  XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
-  XtManageChild(rc2=XmCreateRowColumn(rc,"headrc",al,ac));
-  ac=0;
-  XtManageChild(lb=XmCreateLabel(rc2,"head",al,ac));
-  xs=XmStringCreateLocalized("Head angle: 45.0 degree");
-  XtVaSetValues(lb,XmNlabelString,xs,NULL);
-  XmStringFree(xs);
-  XtVaSetValues(lb,XmNrecomputeSize,FALSE,NULL);
-
   ac=0;
   XtSetArg(al[ac],XmNorientation,XmVERTICAL); ac++;
   XtManageChild(rc=XmCreateRowColumn(rc0,"winrc",al,ac));
   ac=0;
-  XtSetArg(al[ac],XmNminimum,10000); ac++;
-  XtSetArg(al[ac],XmNmaximum,200000); ac++;
+  XtSetArg(al[ac],XmNminimum,10); ac++;
+  XtSetArg(al[ac],XmNmaximum,170); ac++;
   XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
   XtSetArg(al[ac],XmNshowValue,XmNEAR_SLIDER); ac++;
-  XtManageChild(sc=XmCreateScale(rc,"scl",al,ac));
+  XtManageChild(sc=XmCreateScale(rc,"sca",al,ac));
   XtAddCallback(sc,XmNvalueChangedCallback,LegendArrowDialogScaleL,d);
   XtAddCallback(sc,XmNdragCallback,LegendArrowDialogScaleL,d);
   ac=0;
@@ -664,8 +674,8 @@ void LegendArrowDialogSetup(Widget w,void *data,int makewidget)
   XtManageChild(da=XmCreateDrawnButton(rc,"win",al,ac));
   XtAddCallback(da,XmNexposeCallback,LegendArrowDialogPaint,d);
   ac=0;
-  XtSetArg(al[ac],XmNminimum,10000); ac++;
-  XtSetArg(al[ac],XmNmaximum,200000); ac++;
+  XtSetArg(al[ac],XmNminimum,100); ac++;
+  XtSetArg(al[ac],XmNmaximum,2000); ac++;
   XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
   XtSetArg(al[ac],XmNshowValue,XmNEAR_SLIDER); ac++;
   XtManageChild(sc=XmCreateScale(rc,"scw",al,ac));
@@ -678,7 +688,7 @@ void LegendArrowDialogSetup(Widget w,void *data,int makewidget)
 void LegendArrowDialogClose(Widget w,void *data)
 {
   struct LegendArrowDialog *d;
-  int ret;
+  int ret,len;
 
   d=(struct LegendArrowDialog *)data;
   if (d->ret!=IDOK) return;
@@ -691,7 +701,8 @@ void LegendArrowDialogClose(Widget w,void *data)
   if (SetObjFieldFromList(w,"*join",d->Obj,d->Id,"join")) return;
   if (SetObjFieldFromText(w,"*miter",d->Obj,d->Id,"miter_limit")) return;
   if (SetObjFieldFromList(w,"*arrow",d->Obj,d->Id,"arrow")) return;
-  if (putobj(d->Obj,"arrow_length",d->Id,&(d->len))==-1) return;
+  len=d->wid*0.5/tan(d->ang*0.5*MPI/180);
+  if (putobj(d->Obj,"arrow_length",d->Id,&len)==-1) return;
   if (putobj(d->Obj,"arrow_width",d->Id,&(d->wid))==-1) return;
   if (putobj(d->Obj,"R",d->Id,&(d->R))==-1) return;
   if (putobj(d->Obj,"G",d->Id,&(d->G))==-1) return;
@@ -717,6 +728,7 @@ void LegendRectDialogSetupItem(Widget w,struct LegendRectDialog *d,int id)
   SetStyleFromObjField(w,"*style",d->Obj,id,"style");
   SetComboList(w,"*width",cblinewidth,CBLINEWIDTH);
   SetTextFromObjField(GetComboBoxText(w,"*width"),NULL,d->Obj,id,"width");
+  SetComboListFromObjField(GetComboBoxList(w,"*width"),NULL,d->Obj,id,"width");
   getobj(d->Obj,"R",id,0,NULL,&d->R);
   getobj(d->Obj,"G",id,0,NULL,&d->G);
   getobj(d->Obj,"B",id,0,NULL,&d->B);
@@ -891,6 +903,7 @@ void LegendArcDialogSetupItem(Widget w,struct LegendArcDialog *d,int id)
   SetStyleFromObjField(w,"*style",d->Obj,id,"style");
   SetComboList(w,"*width",cblinewidth,CBLINEWIDTH);
   SetTextFromObjField(GetComboBoxText(w,"*width"),NULL,d->Obj,id,"width");
+  SetComboListFromObjField(GetComboBoxList(w,"*width"),NULL,d->Obj,id,"width");
   SetTextFromObjField(w,"*angle1",d->Obj,id,"angle1");
   SetTextFromObjField(w,"*angle2",d->Obj,id,"angle2");
   SetToggleFromObjField(w,"*Pieslice",d->Obj,id,"pieslice");
@@ -1066,9 +1079,11 @@ void LegendMarkDialogSetupItem(Widget w,struct LegendMarkDialog *d,int id)
   MarkDialog(&d->mark,a);
   SetComboList(w,"*size",cbmarksize,CBMARKSIZE);
   SetTextFromObjField(GetComboBoxText(w,"*size"),NULL,d->Obj,id,"size");
+  SetComboListFromObjField(GetComboBoxList(w,"*size"),NULL,d->Obj,id,"size");
   SetStyleFromObjField(w,"*style",d->Obj,id,"style");
   SetComboList(w,"*width",cblinewidth,CBLINEWIDTH);
   SetTextFromObjField(GetComboBoxText(w,"*width"),NULL,d->Obj,id,"width");
+  SetComboListFromObjField(GetComboBoxList(w,"*width"),NULL,d->Obj,id,"width");
   getobj(d->Obj,"R",id,0,NULL,&d->R);
   getobj(d->Obj,"G",id,0,NULL,&d->G);
   getobj(d->Obj,"B",id,0,NULL,&d->B);
@@ -1256,8 +1271,6 @@ void LegendTextDialogSetupItem(Widget w,struct LegendTextDialog *d,int id)
   char *font;
   XmString xs;
   char *buf;
-  unsigned int jis;
-  int i;
 
   SetTextFromObjField(w,"*x",d->Obj,id,"x");
   SetTextFromObjField(w,"*y",d->Obj,id,"y");
@@ -1265,24 +1278,21 @@ void LegendTextDialogSetupItem(Widget w,struct LegendTextDialog *d,int id)
   sgetobjfield(d->Obj,id,"text",NULL,&buf,FALSE,FALSE,FALSE);
 #ifdef JAPANESE
 /* SJIS ---> EUC */
-  for (i=0;i<strlen(buf);i++) {
-    if (niskanji((unsigned char)buf[i])) {
-      jis=njms2jis(((unsigned char)buf[i] << 8)+(unsigned char)buf[i+1]);
-      buf[i]=(jis >> 8) | 0x80;
-      buf[i+1]=(jis & 0xff) | 0x80; 
-      i++;
-    }
-  }
+  njms2euc(buf);
 #endif
   XtVaSetValues(XtNameToWidget(w,"*text"),XmNvalue,buf,NULL);
   memfree(buf);
   SetComboList(w,"*direction",cbdirection,CBDIRECTION);
   SetTextFromObjField(GetComboBoxText(w,"*direction"),NULL,
                       d->Obj,id,"direction");
+  SetComboListFromObjField(GetComboBoxList(w,"*direction"),NULL,
+                      d->Obj,id,"direction");
   SetTextFromObjField(w,"*space",d->Obj,id,"space");
   SetComboList(w,"*pt",cbtextpt,CBTEXTPT);
   SetTextFromObjField(GetComboBoxText(w,"*pt"),NULL,
                       d->Obj,id,"pt");
+  SetComboListFromObjField(GetComboBoxList(w,"*pt"),NULL,
+                      d->Obj,id,"pt");
   SetTextFromObjField(w,"*script",d->Obj,id,"script_size");
   getobj(d->Obj,"font",id,0,NULL,&font);
   XmListDeleteAllItems(GetComboBoxList(w,"*font"));
@@ -1299,6 +1309,7 @@ void LegendTextDialogSetupItem(Widget w,struct LegendTextDialog *d,int id)
     }
     fcur=fcur->next;
   }
+  SetComboBoxVisibleItemCount(w,"*font",j);
   if (selfont!=-1) 
     XmListSelectPos(GetComboBoxList(w,"*font"),selfont+1,TRUE);
 #ifdef JAPANESE
@@ -1317,6 +1328,7 @@ void LegendTextDialogSetupItem(Widget w,struct LegendTextDialog *d,int id)
     }
     fcur=fcur->next;
   }
+  SetComboBoxVisibleItemCount(w,"*jfont",j);
   if (selfont!=-1) 
     XmListSelectPos(GetComboBoxList(w,"*jfont"),selfont+1,TRUE);
 #endif
@@ -1461,8 +1473,6 @@ void LegendTextDialogClose(Widget w,void *data)
   struct LegendTextDialog *d;
   int ret;
   char *buf;
-  int i;
-  unsigned int jms;
   struct fontmap *fcur;
   int *selected,num,a,j;
 
@@ -1476,15 +1486,7 @@ void LegendTextDialogClose(Widget w,void *data)
   XtVaGetValues(XtNameToWidget(w,"*text"),XmNvalue,&buf,NULL);
 #ifdef JAPANESE
 /* EUC ---> SJIS */
-  for (i=0;i<strlen(buf);i++) {
-    if ((buf[i] & 0x80) && (buf[i+1] & 0x80)) {
-      jms=njis2jms((((unsigned char)buf[i] << 8)
-                    +(unsigned char)buf[i+1]) & 0x7f7f);
-      buf[i]=jms >> 8;
-      buf[i+1]=jms & 0xff;
-      i++;
-    }
-  }
+  neuc2jms(buf);
 #endif
   if (sputobjfield(d->Obj,d->Id,"text",buf)!=0) {
     XtFree(buf);
@@ -1561,13 +1563,17 @@ void LegendTextDefDialogSetupItem(Widget w,struct LegendTextDialog *d,int id)
   SetComboList(w,"*direction",cbdirection,CBDIRECTION);
   SetTextFromObjField(GetComboBoxText(w,"*direction"),NULL,
                       d->Obj,id,"direction");
+  SetComboListFromObjField(GetComboBoxList(w,"*direction"),NULL,
+                      d->Obj,id,"direction");
   SetTextFromObjField(w,"*space",d->Obj,id,"space");
   SetComboList(w,"*pt",cbtextpt,CBTEXTPT);
   SetTextFromObjField(GetComboBoxText(w,"*pt"),NULL,
                       d->Obj,id,"pt");
+  SetComboListFromObjField(GetComboBoxList(w,"*pt"),NULL,
+                      d->Obj,id,"pt");
   SetTextFromObjField(w,"*script",d->Obj,id,"script_size");
   getobj(d->Obj,"font",id,0,NULL,&font);
-  XmListDeleteAllItems(XtNameToWidget(w,"*font"));
+  XmListDeleteAllItems(GetComboBoxList(w,"*font"));
   fcur=mxlocal->fontmaproot;
   j=0;
   selfont=-1;
@@ -1581,11 +1587,12 @@ void LegendTextDefDialogSetupItem(Widget w,struct LegendTextDialog *d,int id)
     }
     fcur=fcur->next;
   }
+  SetComboBoxVisibleItemCount(w,"*font",j);
   if (selfont!=-1) 
     XmListSelectPos(GetComboBoxList(w,"*font"),selfont+1,TRUE);
 #ifdef JAPANESE
   getobj(d->Obj,"jfont",id,0,NULL,&font);
-  XmListDeleteAllItems(XtNameToWidget(w,"*jfont"));
+  XmListDeleteAllItems(GetComboBoxList(w,"*jfont"));
   fcur=mxlocal->fontmaproot;
   j=0;
   selfont=-1;
@@ -1599,6 +1606,7 @@ void LegendTextDefDialogSetupItem(Widget w,struct LegendTextDialog *d,int id)
     }
     fcur=fcur->next;
   }
+  SetComboBoxVisibleItemCount(w,"*jfont",j);
   if (selfont!=-1) 
     XmListSelectPos(GetComboBoxList(w,"*jfont"),selfont+1,TRUE);
 #endif
@@ -2256,6 +2264,40 @@ void LegendWinLegendCopy()
   }
 }
 
+void LegendWinLegendCopy2()
+{
+  int sel,i,num;
+  int j,id,perm,type;
+  char *field;
+  struct LegendWin *d;
+
+  d=&(NgraphApp.LegendWin);
+  if (menulock || globallock) return;
+  sel=d->select;
+  num=0;
+  for (i=0;i<LEGENDNUM;i++) {
+    if ((sel>=num) && (sel<=num+d->legend[i])) {
+      if ((id=newobj(d->obj[i]))>=0) {
+        for (j=0;j<chkobjfieldnum(d->obj[i]);j++) {
+          field=chkobjfieldname(d->obj[i],j);
+          perm=chkobjperm(d->obj[i],field);
+          type=chkobjfieldtype(d->obj[i],field);
+          if ((strcmp2(field,"name")!=0)
+            && ((perm&NREAD)!=0) && ((perm&NWRITE)!=0) && (type<NVFUNC))
+            copyobj(d->obj[i],field,id,sel-num);
+        }
+        d->legend[i]++;
+        for (j=d->legend[i];j>sel-num+1;j--) moveupobj(d->obj[i],j);
+        NgraphApp.Changed=TRUE;
+      }
+      d->select=sel;
+      LegendWinUpdate(FALSE);
+      break;
+    }
+    num+=d->legend[i]+1;
+  }
+}
+
 void LegendWinLegendTop()
 {
   int sel,i,num;
@@ -2633,7 +2675,13 @@ void LegendWinExpose(Widget wi,XtPointer client_data,XtPointer call_data)
           getobj(d->obj[k],"text",i,0,NULL,&text);
           rect.x=x;
           rect.width=25*w;
-          len=sprintf(buf,"%.25s",text);
+          strncpy(buf, text, 25);
+          buf[25] = '\0';
+          len = strlen(buf);
+#ifdef JAPANESE
+/* SJIS ---> EUC */
+          njms2euc(buf);
+#endif
           ExtTextOut(disp,win,gc,x,y,&rect,buf,len);
           x+=25*w;
           break;
@@ -2702,6 +2750,7 @@ void LegendWinEvRButtonDown(unsigned int state,TPoint point,
     XtSetSensitive(XtNameToWidget(d->popup,"button_6"),True);
     XtSetSensitive(XtNameToWidget(d->popup,"button_7"),True);
     XtSetSensitive(XtNameToWidget(d->popup,"button_8"),True);
+    XtSetSensitive(XtNameToWidget(d->popup,"button_9"),True);
   } else {
     XtSetSensitive(XtNameToWidget(d->popup,"button_0"),False);
     XtSetSensitive(XtNameToWidget(d->popup,"button_1"),False);
@@ -2712,6 +2761,7 @@ void LegendWinEvRButtonDown(unsigned int state,TPoint point,
     XtSetSensitive(XtNameToWidget(d->popup,"button_6"),False);
     XtSetSensitive(XtNameToWidget(d->popup,"button_7"),False);
     XtSetSensitive(XtNameToWidget(d->popup,"button_8"),False);
+    XtSetSensitive(XtNameToWidget(d->popup,"button_9"),False);
   }
   XmMenuPosition(d->popup,event);
   XtManageChild(d->popup);
@@ -2775,6 +2825,9 @@ void LegendWinPopupMenu(Widget w,XtPointer client_data,XtPointer call_data)
     LegendWinLegendCopy();
     break;
   case 8: 
+    LegendWinLegendCopy2();
+    break;
+  case 9: 
     LegendWinLegendHidden();
     break;
   }
@@ -2823,7 +2876,8 @@ void LegendWinEvKeyDown(Widget w,XtPointer client_data,XEvent *event,
     LegendWinLegendDelete();
     break;
   case XK_Insert:
-    LegendWinLegendCopy();
+    if (e->state & ShiftMask) LegendWinLegendCopy2();
+    else LegendWinLegendCopy();
     break;
   case XK_Home:
     if (e->state & ShiftMask) LegendWinLegendTop();
@@ -2847,12 +2901,14 @@ void LegendWindowUnmap(Widget w,XtPointer client_data,XtPointer call_data)
 {
   struct LegendWin *d;
   Position x,y,x0,y0;
+  Dimension w0,h0;
 
   d=&(NgraphApp.LegendWin);
   if (d->Win!=NULL) {
     XtVaGetValues(d->Win,XmNx,&x,XmNy,&y,
-                         XmNwidth,&(menulocal.legendwidth),
-                         XmNheight,&(menulocal.legendheight),NULL);
+                         XmNwidth,&w0,XmNheight,&h0,NULL);
+    menulocal.legendwidth=w0;
+    menulocal.legendheight=h0;
     XtTranslateCoords(TopLevel,0,0,&x0,&y0);
     menulocal.legendx=x-x0;
     menulocal.legendy=y-y0;
@@ -2941,6 +2997,7 @@ void CmLegendWindow(Widget w,XtPointer client_data,XtPointer call_data)
     XmVaPUSHBUTTON,NULL,NULL,NULL,NULL,
     XmVaPUSHBUTTON,NULL,NULL,NULL,NULL,
     XmVaPUSHBUTTON,NULL,NULL,NULL,NULL,
+    XmVaPUSHBUTTON,NULL,NULL,NULL,NULL,
     NULL);
     XtAddEventHandler(d->text,KeyPressMask,False,LegendWinEvKeyDown,d);
     XmToggleButtonSetState(XtNameToWidget(TopLevel,"*windowmenu.button_2"),
