@@ -1,21 +1,38 @@
-/**
- *
- * $Id: x11lgndx.c,v 1.2 1999/04/11 06:09:20 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11lgndx.c,v 1.5 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11lgndx.c,v $
+ * Revision 1.5  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.4  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.3  2001/03/23 12:17:43  isizaka
+ * for 6.3.13
+ *
  * Revision 1.2  1999/04/11 06:09:20  isizaka
  * *** empty log message ***
  *
@@ -49,6 +66,7 @@ void LegendGaussDialogSetupItem(Widget w,struct LegendGaussDialog *d,int id)
   SetStyleFromObjField(w,"*style",d->Obj,id,"style");
   SetComboList(w,"*width",cblinewidth,CBLINEWIDTH);
   SetTextFromObjField(GetComboBoxText(w,"*width"),NULL,d->Obj,id,"width");
+  SetComboListFromObjField(GetComboBoxList(w,"*width"),NULL,d->Obj,id,"width");
   SetListFromObjField(w,"*join",d->Obj,id,"join");
   SetTextFromObjField(w,"*miter",d->Obj,id,"miter_limit");
   getobj(d->Obj,"R",id,0,NULL,&d->R);
@@ -76,8 +94,8 @@ void LegendGaussDialogSetupItem(Widget w,struct LegendGaussDialog *d,int id)
   else if (d->Mode==3) 
     XmToggleButtonSetState(XtNameToWidget(w,"*Sin"),TRUE,TRUE);
   XtVaSetValues(XtNameToWidget(w,"*div"),XmNvalue,d->Div,NULL);
-  XtVaSetValues(XtNameToWidget(w,"*sch"),XmNvalue,round(d->Position*100),NULL);
-  XtVaSetValues(XtNameToWidget(w,"*scv"),XmNvalue,round(d->Param*1000),NULL);
+  XtVaSetValues(XtNameToWidget(w,"*sch"),XmNvalue,nround(d->Position*100),NULL);
+  XtVaSetValues(XtNameToWidget(w,"*scv"),XmNvalue,nround(d->Param*1000),NULL);
 }
 
 void LegendGaussDialogPaint(Widget w,XtPointer client_data,XtPointer call_data)
@@ -109,7 +127,7 @@ void LegendGaussDialogPaint(Widget w,XtPointer client_data,XtPointer call_data)
   if (d->Wdx<d->Wdy) dw=d->Wdy;
   else dw=d->Wdx;
   ppd=pw/((double )dw);
-  dpi=round(ppd*2540);
+  dpi=nround(ppd*2540);
   minx=ww/2-d->Wdx*ppd/2;
   miny=hh/2-d->Wdy*ppd/2;
   maxx=ww/2+d->Wdx*ppd/2;
@@ -153,17 +171,17 @@ void LegendGaussDialogPaint(Widget w,XtPointer client_data,XtPointer call_data)
           y=Amp*0.5*(sin(2.0*MPI*(tmp-d->Position*0.5))+1);
         }
         if (d->Dir==0) {
-          spx[i]=round(x);
-          spy[i]=d->Wdy-round(y);
+          spx[i]=nround(x);
+          spy[i]=d->Wdy-nround(y);
         } else if (d->Dir==1) {
-          spx[i]=round(x);
-          spy[i]=round(y);
+          spx[i]=nround(x);
+          spy[i]=nround(y);
         } else if (d->Dir==2) {
-          spx[i]=d->Wdx-round(y);
-          spy[i]=d->Wdy-round(x);
+          spx[i]=d->Wdx-nround(y);
+          spy[i]=d->Wdy-nround(x);
         } else if (d->Dir==3) {
-          spx[i]=round(y);
-          spy[i]=d->Wdy-round(x);
+          spx[i]=nround(y);
+          spy[i]=d->Wdy-nround(x);
         }
         spz[i]=i;
       }
@@ -452,17 +470,17 @@ void LegendGaussDialogClose(Widget w,void *data)
       y=Amp*0.5*(sin(2.0*MPI*(tmp-d->Position*0.5))+1);
     }
     if (d->Dir==0) {
-      gx=round(x);
-      gy=d->Wdy-round(y);
+      gx=nround(x);
+      gy=d->Wdy-nround(y);
     } else if (d->Dir==1) {
-      gx=round(x);
-      gy=round(y);
+      gx=nround(x);
+      gy=nround(y);
     } else if (d->Dir==2) {
-      gx=d->Wdx-round(y);
-      gy=d->Wdy-round(x);
+      gx=d->Wdx-nround(y);
+      gy=d->Wdy-nround(x);
     } else if (d->Dir==3) {
-      gx=round(y);
-      gy=d->Wdy-round(x);
+      gx=nround(y);
+      gy=d->Wdy-nround(x);
     }
     gx+=d->Minx;
     gy+=d->Miny;
