@@ -1,21 +1,41 @@
-/**
- *
- * $Id: x11menu.c,v 1.5 1999/05/08 13:31:30 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11menu.c,v 1.9 2002/08/15 13:12:44 isizaka Exp $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11menu.c,v $
+ * Revision 1.9  2002/08/15 13:12:44  isizaka
+ * buf fix: XmNwidth and XmNheight (thanks to Mr tanaka, tt@flab.fujitsu.co.jp)
+ *
+ * Revision 1.8  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.7  2001/07/14 17:44:50  isizaka
+ * for 6.03.14
+ *
+ * Revision 1.6  2001/03/23 12:17:43  isizaka
+ * for 6.3.13
+ *
  * Revision 1.5  1999/05/08 13:31:30  isizaka
  * for release 6.03.02
  *
@@ -124,7 +144,8 @@ unsigned int black,white,gray,red,blue;
 
 void childhandler(int sig)
 {
-  int i,num,*data;
+  int i,num;
+  pid_t *data;
 
   if (!signaltrap) return;
   signal(sig,SIG_IGN);
@@ -389,7 +410,8 @@ void DropTransfer(Widget w,XtPointer closure,Atom *seltype,
           LoadPrmFile(name);
           load=TRUE;
         } else if ((strcmp0(ext,"NGP")==0) || (strcmp0(ext,"ngp")==0)) {
-          LoadNgpFile(name,FALSE,TRUE,"./",FALSE,NULL);
+          LoadNgpFile(name,menulocal.ignorepath,menulocal.expand,
+                      menulocal.expanddir,FALSE,NULL);
           load=TRUE;
         }
       }
@@ -929,7 +951,7 @@ void createmarkpixmap(Window win)
   G2=B2=255;
   for (i=0;i<90;i++) {
     pix=NgraphApp.markpix[i]=XCreatePixmap(Disp,win,20,20,depth);
-    dpi=round(20*25.4);
+    dpi=nround(20*25.4);
     XGC=XCreateGC(Disp,pix,0,0);
     XSetForeground(Disp,XGC,RGB(255,255,255));
     XFillRectangle(Disp,pix,XGC,0,0,20,20);
@@ -954,29 +976,31 @@ void createicon(Window win)
   unsigned int width,height,border,depth;
   Window root;
   GC gc;
-  int i,h,w;
+  int h,w;
   char c;
 
   XGetGeometry(Disp,win,&root,&x,&y,&width,&height,&border,&depth);
-  pix=XCreatePixmap(Disp,win,icon_width,icon_height,depth);
+  pix=XCreatePixmap(Disp,win,icon_width+2,icon_height+2,depth);
   gc=XCreateGC(Disp,pix,0,0);
-  for (i=0;i<10;i++) {
-    switch(i) {
-    case 0: c='.'; XSetForeground(Disp,gc,RGB(0,0,0)); break;
-    case 1: c='#'; XSetForeground(Disp,gc,RGB(0,0,255)); break;
-    case 2: c='a'; XSetForeground(Disp,gc,RGB(0,127,127)); break;
-    case 3: c='b'; XSetForeground(Disp,gc,RGB(0,255,255)); break;
-    case 4: c='c'; XSetForeground(Disp,gc,RGB(127,127,0)); break;
-    case 5: c='d'; XSetForeground(Disp,gc,RGB(127,127,127)); break;
-    case 6: c='e'; XSetForeground(Disp,gc,RGB(191,191,191)); break;
-    case 7: c='f'; XSetForeground(Disp,gc,RGB(255,0,0)); break;
-    case 8: c='g'; XSetForeground(Disp,gc,RGB(255,255,0)); break;
-    case 9: c='h'; XSetForeground(Disp,gc,RGB(255,255,255)); break;
-    default: c='.'; XSetForeground(Disp,gc,RGB(0,0,0)); break;
+  for (h=0;h<icon_height+2;h++) {
+    for (w=0;w<icon_width+2;w++) {
+      if ((h==0) || (h==icon_height+1) || (w==0) || (w==icon_width+1)) c='a';
+      else c=icon_xpm[h-1][w-1];
+      switch(c) {
+      case '.': XSetForeground(Disp,gc,RGB(0,0,0)); break;
+      case '#': XSetForeground(Disp,gc,RGB(0,0,255)); break;
+      case 'a': XSetForeground(Disp,gc,RGB(0,127,127)); break;
+      case 'b': XSetForeground(Disp,gc,RGB(0,255,255)); break;
+      case 'c': XSetForeground(Disp,gc,RGB(127,127,0)); break;
+      case 'd': XSetForeground(Disp,gc,RGB(127,127,127)); break;
+      case 'e': XSetForeground(Disp,gc,RGB(191,191,191)); break;
+      case 'f': XSetForeground(Disp,gc,RGB(255,0,0)); break;
+      case 'g': XSetForeground(Disp,gc,RGB(255,255,0)); break;
+      case 'h': XSetForeground(Disp,gc,RGB(255,255,255)); break;
+      default: XSetForeground(Disp,gc,RGB(0,0,0)); break;
+      }
+      XDrawPoint(Disp,pix,gc,w,h);
     }
-    for (h=0;h<icon_height;h++)
-      for (w=0;w<icon_width;w++)
-        if (c==icon_xpm[h][w]) XDrawPoint(Disp,pix,gc,w,h);
   }
   XFreeGC(Disp,gc);
   NgraphApp.iconpix=pix;
@@ -1172,7 +1196,12 @@ int createcolormap()
   Colormap cmap;
 
   depth=mxlocal->cdepth;
+#ifdef PRIVATECOLOR
+  cmap=XCreateColormap(Disp,RootWindow(Disp,0),
+                       DefaultVisual(Disp,0),AllocNone);
+#else
   cmap=DefaultColormap(Disp,0);
+#endif
   mxlocal->cmap=cmap;
   return TRUE;
 }
@@ -1435,6 +1464,7 @@ void application(char *file)
   }
 
   XtRealizeWidget(TopLevel);
+  XSetWindowColormap(XtDisplay(TopLevel),XtWindow(TopLevel),mxlocal->cmap);
 
   if ((menulocal.framex==0) && (menulocal.framey==0)) GetWMFrame(TopLevel);
   defaultwindowconfig();
@@ -1475,13 +1505,14 @@ void application(char *file)
       if ((strcmp0(ext,"PRM")==0) || (strcmp0(ext,"prm")==0))
         LoadPrmFile(file); 
       else if ((strcmp0(ext,"NGP")==0) || (strcmp0(ext,"ngp")==0))
-        LoadNgpFile(file,FALSE,TRUE,"./",FALSE,NULL);
+        LoadNgpFile(file,menulocal.ignorepath,menulocal.expand,
+                         menulocal.expanddir,FALSE,NULL);
     }
   }
 
   CmViewerDrawB(NgraphApp.Viewer.Win,NULL,NULL);
 
-  arrayinit(&childlist,sizeof(int));
+  arrayinit(&childlist,sizeof(pid_t));
   signaltrap=TRUE;
   signal(SIGCHLD,childhandler); 
 
@@ -1744,6 +1775,7 @@ int InputYN(char *mes)
 void CmReloadWindowConfig()
 {
   Position x,y;
+  Dimension w0,h0;
 
   if (NgraphApp.FileWin.Win!=NULL) CmFileWindow(NULL,NULL,NULL);
   if (NgraphApp.AxisWin.Win!=NULL) CmAxisWindow(NULL,NULL,NULL);
@@ -1756,10 +1788,11 @@ void CmReloadWindowConfig()
   mx11windowconfig();
   XtVaGetValues(MainWin,
                 XmNx,&x,XmNy,&y,
-                XmNwidth,&(menulocal.menuwidth),
-                XmNheight,&(menulocal.menuheight),NULL);
+                XmNwidth,&w0,XmNheight,&h0,NULL);
   menulocal.menux=x;
   menulocal.menuy=y;
+  menulocal.menuwidth=w0;
+  menulocal.menuheight=h0;
   defaultwindowconfig();
   if (menulocal.dialogopen) CmInformationWindow(NULL,NULL,NULL);
   if (menulocal.coordopen) CmCoordinateWindow(NULL,NULL,NULL);
