@@ -1,21 +1,35 @@
-/**
- *
- * $Id: x11merge.c,v 1.5 1999/05/08 13:31:30 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11merge.c,v 1.7 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11merge.c,v $
+ * Revision 1.7  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.6  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
  * Revision 1.5  1999/05/08 13:31:30  isizaka
  * for release 6.03.02
  *
@@ -61,7 +75,9 @@ void MergeDialogSetupItem(Widget w,struct MergeDialog *d,int file,int id)
   SetTextFromObjField(w,"*topmargin",d->Obj,id,"top_margin");
   SetTextFromObjField(w,"*leftmargin",d->Obj,id,"left_margin");
   SetTextFromObjField(w,"*zoom",d->Obj,id,"zoom");
+#ifdef JAPANESE
   SetToggleFromObjField(w,"*GreekSymbol",d->Obj,id,"symbol_greek");
+#endif
 }
 
 void MergeDialogDelete(Widget w,XtPointer client_data,XtPointer call_data)
@@ -134,8 +150,10 @@ void MergeDialogSetup(Widget w,void *data,int makewidget)
   XtManageChild(XmCreateLabel(rc2,"Zoom",al,ac));
   ac=0;
   XtManageChild(XmCreateTextField(rc2,"zoom",al,ac));
+#ifdef JAPANESE
   ac=0;
   XtManageChild(XmCreateToggleButton(rc2,"GreekSymbol",al,ac));
+#endif
   }
   MergeDialogSetupItem(w,d,TRUE,d->Id);
 }
@@ -153,8 +171,10 @@ void MergeDialogClose(Widget w,void *data)
   if (SetObjFieldFromText(w,"*topmargin",d->Obj,d->Id,"top_margin")) return;
   if (SetObjFieldFromText(w,"*leftmargin",d->Obj,d->Id,"left_margin")) return;
   if (SetObjFieldFromText(w,"*zoom",d->Obj,d->Id,"zoom")) return;
+#ifdef JAPANESE
   if (SetObjFieldFromToggle(w,"*GreekSymbol",
       d->Obj,d->Id,"symbol_greek")) return;
+#endif
   d->ret=ret;
 }
 
@@ -731,12 +751,14 @@ void MergeWindowUnmap(Widget w,XtPointer client_data,XtPointer call_data)
 {
   struct MergeWin *d;
   Position x,y,x0,y0;
+  Dimension w0,h0;
 
   d=&(NgraphApp.MergeWin);
   if (d->Win!=NULL) {
     XtVaGetValues(d->Win,XmNx,&x,XmNy,&y,
-                         XmNwidth,&(menulocal.mergewidth),
-                         XmNheight,&(menulocal.mergeheight),NULL);
+                         XmNwidth,&w0,XmNheight,&h0,NULL);
+    menulocal.mergewidth=w0;
+    menulocal.mergeheight=h0;
     XtTranslateCoords(TopLevel,0,0,&x0,&y0);
     menulocal.mergex=x-x0;
     menulocal.mergey=y-y0;
